! =================================================================================================================================
! MODULE       : sapiens_lcchange
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Impact of land cover change on carbon stocks
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_lcchange.f90 $
!! $Date: 2013-07-03 10:16:15 +0200 (Wed, 03 Jul 2013) $
!! $Revision: 1356 $
!! \n
!_ ================================================================================================================================


MODULE sapiens_lcchange

  ! modules used:
  
  USE ioipsl_para
  USE stomate_data
  USE pft_parameters
  USE constantes
  USE stomate_prescribe
  USE function_library,    ONLY: wood_to_dia, wood_to_ba, &
                                 check_biomass_sync, check_surface_area, &
                                 check_mass_balance 
  
  IMPLICIT NONE
  
  PRIVATE
  PUBLIC land_cover_change_main, age_class_distr, merge_biomass_pfts

CONTAINS


! ================================================================================================================================
!! SUBROUTINE   : land_cover_change_main
!!
!>\BRIEF        Impact of land cover change on carbon stocks
!!
!! DESCRIPTION  : This subroutine is activated by setting VEGET_UPDATE to 1Y 
!! in the sechiba.card file. Subsequently, the impact of land cover change on 
!! in-situ carbon stocks is computed in this subroutine. Fluxes from products are
!! now dealt with in a separate subroutine CHECK. The land cover change is read from
!! driver maps containing the new "maximal" coverage fraction of a PFT (::veget_cov_max). 
!! On the basis of this difference, the amount of 'new establishment'/'biomass export',
!! and increase/decrease of each component, are estimated. The soil carbon from the
!! PFT which experienced a decrease in ::veget_cov_max is pooled and used as the initial
!! soil carbon for the PFTs that experienced an increase in ::veget_cov_max.\n
!!
!! RECENT CHANGE(S) : (1) Age-classes were introduced and (2) product use was moved 
!! to a separate module
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE land_cover_change_main (npts, dt_days, veget_cov_max, veget_cov, &
       harvest_pool, harvest_type, harvest_cut, harvest_area, &
       litter, som, lignin_struc, lignin_wood, &
       PFTpresent, everywhere, when_growthinit, & 
       leaf_frac, circ_class_n, circ_class_biomass, &
       atm_to_bm, forest_managed, KF, wstress_season, &
       wstress_month, plant_status, npp_longterm, &
       age, lm_lastyearmax, harvest_pool_bound, resolution, &
       bm_to_litter,turnover_daily, leaf_age, circ_class_dist, &
       tau_eff_leaf, tau_eff_sap, &
       tau_eff_root, veget_cov_max_new, age_stand, last_cut, &
       k_latosa_adapt, losses, light_tran_to_level_season,&
       EndOfYear, lpft_replant, soil_n_min, bact, species_change_map)

    IMPLICIT NONE
    
 !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
    ! NOTE: if harvest_pool_bound is not explicitely defined 
    ! the model uses 1:ndia_harvest+2 as its dimensions. If 
    ! you like to do so, remember to change the counters in 
    ! this routine
    INTEGER, INTENT(in)                              :: npts                   !! Domain size - number of pixels (unitless)
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)      :: forest_managed         !! forest management flag
    REAL(r_std), INTENT(in)                          :: dt_days                !! Time step of vegetation dynamics for stomate (days)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: circ_class_dist        !! The probability distribution of trees in a circ
                                                                               !! class in case of a redistribution (unitless).
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget_cov              !! Fraction of vegetation type including 
                                                                               !! non-biological fraction (unitless)   
    REAL(r_std), DIMENSION(npts,2), INTENT(in)       :: resolution             !! Resolution at each grid point (m) [1=E-W, 2=N-S]
    REAL(r_std), DIMENSION(0:ndia_harvest+1), &
                                       INTENT(in)    :: harvest_pool_bound     !! The boundaries of the diameter classes
                                                                               !! in the wood harvest pools @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root           !! Effective root turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap            !! Effective sapwood turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf           !! Effective leaf turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)      :: veget_cov_max_new          !! new "maximal" coverage fraction of a PFT on the  
                                                                               !! ground.  May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)
    CHARACTER(15), INTENT(in)                         :: losses                !! Flag that determines how the losses are 
                                                                               !! distributed. There are currently two options
                                                                               !! 'proportional' and 'youngest'
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)          :: lpft_replant          !! Set to true if a PFT has been clearcut
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)       :: species_change_map    !! A map which gives the PFT number that each PFT

    
    REAL(r_std), DIMENSION(:,:,:),INTENT(in)          :: light_tran_to_level_season !! Mean seasonal fraction of light transmitted
    LOGICAL, INTENT(in)                               :: EndOfYear             !! Flag set on the last day of the year used 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: soil_n_min          !!mineral nitrogen in the soil (gN/m**2)  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: bact                !! denitrifier biomass (gC/m**2)
                                                                              
    !! 0.3 Modified variables   
    
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: PFTpresent             !! PFT present (0 or 1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: plant_status           !! Growth and phenological status of the plant
                                                                               !! istatus = Phases defined in constantes
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: KF                     !! Scaling factor to convert sapwood mass into leaf 
                                                                               !! mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: k_latosa_adapt         !! Leaf to sapwood area adapted for long 
                                                                               !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_cov_max              !! "maximal" coverage fraction of a PFT on the ground
                                                                               !! May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: atm_to_bm              !! C and N taken from the atmosphere to get C and N to  
                                                                               !! create the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: everywhere             !! is the PFT everywhere in the grid box or 
                                                                               !! very localized (after its introduction) (?)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: when_growthinit        !! how many days ago was the beginning of 
                                                                               !! the growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season         !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_month          !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_longterm           !! "long term" net primary productivity
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                    !! mean age (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lm_lastyearmax         !! last year's maximum leaf mass for each PFT 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_struc           !! ratio Lignine/Carbon in structural litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_wood            !! ratio Lignine/Carbon in woody litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: som                  !! carbon pool: active, slow, or passive 
                                                                               !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                 !! metabolic and structural litter, above and 
                                                                               !! below ground @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n           !! Number of individuals in each circ class
                                                                               !! @tex $(ind m^{-2})$ @endtex   
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac              !! fraction of leaves in leaf age class (unitless;0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age               !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: bm_to_litter           !! Transfer of biomass to litter 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     ::turnover_daily          !! Turnover rates            
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass     !! Biomass components of the model tree within a 
                                                                               !! circumference class @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: harvest_pool           !! The wood and biomass that have been
                                                                               !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_type           !! Type of management that resulted
                                                                               !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_cut            !! Type of cutting that was used for the harvest
                                                                               !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_area           !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: age_stand              !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: last_cut               !! Years since last thinning (years)
    !! 0.4 Local variables

    INTEGER(i_std)                                   :: ipts,ivm, imbc         !! Indices (unitless)
    INTEGER(i_std)                                   :: igroup,ifm             !! Indices (unitless)
    INTEGER(i_std)                                   :: idia,icir,ilage        !! Indices (unitless)
    INTEGER(i_std)                                   :: jcir,ilit,ilev         !! Indices (unitless)
    INTEGER(i_std)                                   :: ipar,iele,icarb        !! Indices (unitless)
    INTEGER(i_std)                                   :: ivma,iyoung,jvma       !! Indices (unitless)
    INTEGER(i_std)                                   :: nagec_temp,ispec       !! Temporary counter
    LOGICAL                                          :: lsync                  !! returns false if the biomass is not synced
    REAL(r_std)                                      :: total_losses           !! Sum of losses only in delta_veget_cov (unitless, 0-1)
    REAL(r_std)                                      :: share_expanded         !! Share of the veget_cov_max of the existing vegetation
                                                                               !! within a PFT over the total veget_cov_max following 
                                                                               !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(npts)                     :: pixel_area             !! surface area of the pixel @tex ($m^{2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvmap)               :: veget_cov_max_group        !! Total veget_cov_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap)               :: veget_cov_max_new_group    !! Total veget_cov_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap)               :: delta_veget_cov            !! Change in veget_cov_max due to LCC restricted to the
                                                                               !! PFT groups thus not accounting for the age-classes
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: loss_gain              !! The same as delta_veget_cov but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)
    REAL(r_std), DIMENSION(npts)                     :: loss_gain_nobio        !! The change in nobio fraction during the LCC.  Right
                                                                               !! now we only take into account a single nobio type.
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(nlitt,nlevs,nelements)    :: litter_bank            !! Bank to store the litter that becomes available when
                                                                               !! part of a PFT is removed @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(ncarb,nelements)          :: soil_bank              !! Bank to store soil carbon that becomes available when
                                                                               !! part of a PFT is removed  @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: struct_ltr_bank        !! Bank to store the lignin in structural litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nlevs)                     :: woody_ltr_bank         !! Bank to store the lignin in woody litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nvm,nparts,nelements)      :: fresh_litter           !! Pool of fresh litter that is being released during 
    REAL(r_std),DIMENSION(nvm,nparts,nelements)      :: fresh_som              !! Pool of fresh som that is being released during
                                                                               !! LCC @tex ($gC m^{-2}$) @endtex 
    REAL(r_std),DIMENSION(nparts,nelements)          :: fresh_ltr_bank         !! Bank to store all the fresh litter from site
    REAL(r_std),DIMENSION(nparts,nelements)          :: fresh_som_bank         !! Bank to store all the fresh som from site 
    REAL(r_std)                                      :: bact_bank              !! Bank to
    REAL(r_std), DIMENSION(nnspec)                   :: soil_n_min_bank        !! Bank to 
                                                                               !! clearing during LCC. Weighted value of fresh_litter
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: atm_to_bm_old          !!

                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std)                                      :: moi_bank_season        !! bank to store moiavail_growingseason of the cleared
                                                                               !! PFT's (unitless; 0-1)
    REAL(r_std)                                      :: moi_bank_month         !! bank to store moiavail_monthly of the cleared
                                                                               !! PFT's (unitless; 0-1)
    REAL(r_std)                                      :: trees_needed           !! Number of trees leftthat should be present in a 
                                                                               !! circumference class when merging the available and 
                                                                               !! newly established vegetation
                                                                               !! @tex $(ind m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                    :: diameters_temp         !! Temporary variable to store the diameters of the
                                                                               !! different diameter classes (m)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: new_biomass            !! Temporary variable for biomass 
                                                                               !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nleafages)       :: new_leaf_frac          !! Temporary variable for leaf_frac (unitless;0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_ind                !! Temporary variable for ind @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ_class_n       !! Temporary variable for circ_class_n of the
                                                                               !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: est_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements) &
                                                     :: new_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! new vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ               !! Temporary variable for circ (m)
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: new_atm_to_bm          !! Temporary variable for atm_to_bm 
                                                                               !! @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_age                !! Temporary variable for age (years)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_lm_lastyearmax     !! Temporary variable for lm_last_year 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_everywhere         !! Temporary variable for everywhere (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_when_growthinit    !! Dummy for when_growthinit (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_npp_longterm       !! Dummy for npp_long_term
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) &
                                                     :: check_intern           !! Contains the components of the internal
                                                                               !! mass balance chech for this routine
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern         !! Check closure of internal mass balance
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start             !! Start and end pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_end               !! Start and end pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_PFTpresent         !! Dummy for PFTpresent (0 or 1)
    REAL, DIMENSION(npts,nvm)                        :: dum_plant_status       !! Dummy for plant_status
    REAL(r_std)                                      :: harvest_ratio_stem     !! The fraction of the reserve pools which are
                                                                               !! located in the stem (unitless, 0-1)
    REAL(r_std)                                      :: harvest_ratio_branch   !! The fraction of the reserve pools which are
                                                                               !! located in the branches/roots (unitless, 0-1)
    REAL(r_std)                                      :: harvest_ratio_litter   !! The fraction of the reserve pools which are
                                                                               !! moved to the litter (unitless, 0-1)
    REAL(r_std)                                      :: total_nonheart_biomass !! The total amount of non-heartwood biomass
                                                                               !! in a model tree
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std)                                      :: total_nonheart_biomass_stem !! The total amount of non-heartwood biomass
                                                                               !! in a model tree stem
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std)                                      :: total_nonheart_biomass_branch !! The total amount of non-heartwood biomass
                                                                               !! in a model tree branches/roots
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std)                                      :: temp_sum,temp_total
    REAL(r_std), DIMENSION(npts)                     :: change_bio             !! The overall change in vegetative fraction for
                                                                               !! this pixel (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                     :: change_nobio           !! The overall change in non-vegetative fraction for
                                                                               !! this pixel (0-1, unitless)
    REAL(r_std), DIMENSION(nlitt,nlevs)              :: litter_weight_expanded !! The fraction of litter on the expanded
                                                                               !! PFT.
                                                                               !! @tex $-$ @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: litter_weight_struc    !! A weighting factor for the structural litter.
                                                                               !! (unitless)
    REAL(r_std), DIMENSION(nlevs)                    :: litter_weight_woody    !! A weighting factor for the woody litter.
                                                                               !! (unitless)
    LOGICAL                                          :: lconverged             !! A flag for loop convergence
    INTEGER(i_std)                                   :: max_pft                !! The number of the PFT with the largest addition
                                                                               !! in land area.
    INTEGER(i_std)                                   :: min_pft                !! The number of the PFT with the largest reduction
                                                                               !! in land area (most negative value).
    INTEGER(i_std)                                   :: nsteps                 !! The number of convergence steps tried.
    INTEGER(i_std)                                   :: ncount                 !! The number of PFTs with significant changes
                                                                               !! in vegetation.
    INTEGER(i_std)                                   :: nagec_local            !! The number of age classes in this map PFT.
    REAL(r_std)                                      :: difference             !! How much leftover veget_cov_max we have
    REAL(r_std)                                      :: temp_sum_old           !! 
    REAL(r_std)                                      :: temp_sum_new           !! 
    REAL(r_std), DIMENSION(npts,nvm)                 :: veget_cov_max_new_adj      !! new "maximal" coverage fraction of a PFT on the  
                                                                               !! ground adjusted for numerical precision.  
                                                                               !! May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: veget_cov_max_begin    !! Temporary variable to check area conservation 
    REAL(r_std), DIMENSION(npts,nvm,ndia_harvest+1,nelements) &
                                                     :: harvest_pool_old       !! Temporary variable to check mass conservation 
  
    !+++ TEMP++++
    REAL(r_std), DIMENSION(npts,nnobio)              :: frac_nobio_start,frac_nobio_end
    REAL(r_std)                                      :: min_delta_veget_cov
    REAL(r_std), DIMENSION(nelements,2)              :: test_mass              !! Temporary variable to check mass conservation 
    REAL(r_std), DIMENSION(nelements)                :: test_mass_bank         !! Temporary variable to check mass conservation
    REAL(r_std), DIMENSION(3,nelements,2)            :: test_bank              !! Temporary variable to check mass conservation
!_ ================================================================================================================================

    IF (printlev.GE.3) WRITE(numout,*) 'Entering land cover change main'

    !  +++COMMENT+++
    !  This part is poorly coded as it uses the externalisation to
    !  implement the age classes. The advanatage of this approach is
    !  that is rather simple to implement. The alternative requires
    !  that a dimension for age classes is added to almost all fluxes
    !  and pool in sechiba and stomate. Also the externalisation would
    !  need to be adjusted as the parameters are now used for the
    !  different age classes.
    !
    !  The coding is considered poor because it is : (1) sensitive to 
    !  the order in which the PFTs are defined and (2) inconsistent as
    !  cohorts (within a stand got an own dimension) and age classes
    !  (across stands) are implemented through the externalisation.
    !  An example for 2 age classes and 8 tree PFTs
    !  PFT_1: bare soil
    !  PFT_2: PFT2_1 age class 1 (youngest)
    !  PFT_3: PFT2_2 age class 2 (youngest + 1)
    !  PFT_4: PFT3_1 age class 1
    !  PFT_5: PFT3_2 age class 2
    !  ...
    !  PFT_18: PFT 10 agricultural PFTs don't have age classes
    !  PFT_19: PFT 11 agricultural PFTs don't have age classes
    !  PFT_20: PFT 11 agricultural PFTs don't have age classes
    !  PFT_21: PFT 11 agricultural PFTs don't have age classes
    !  
    !  A PFT group are all age classes a a certain PFT/species
    !
    !  Some of this complexity could be avoided by preparing the maps
    !  for the exact number of age classes we want to deal with
    !  but that just shifts the complexity. It also means that we have
    !  to prescribe the age classes, which is not our goal. 
    !  For now it is assumed the maps only consider the PFTs without 
    !  age classes. So part of this code needs to conver the maps
    !  without age classes into variables in ORCHIDEE with age classes.
    !  +++++++++++

 !! 1. initial checks

    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('land_cover')

    ! Check for problematic situations in which the frac_nobio
    ! in a pixel is larger than the pixel itself.
    frac_nobio_start(:,:) = zero
    frac_nobio_end(:,:) = zero

    DO ipts=1,npts

       frac_nobio_start(ipts,1)=un-SUM(veget_cov_max(ipts,:))

       IF(frac_nobio_start(ipts,1) .GT. un)THEN

          WRITE(numout,*) 'ERROR: nobio fraction greater than one in LCC!'
          WRITE(numout,*) 'ipts, frac_nobio_start(ipts,1): ',ipts, &
               frac_nobio_start(ipts,1)
          DO ivm=1,nvm
             WRITE(numout,*) ivm,veget_cov_max(ipts,ivm)
          ENDDO
          CALL ipslerr (3,'land_cover_change_main', &
               'nobio fraction greater than one',&
               'Check the PFT maps you are using!','')

       ENDIF

    ENDDO
    
    ! Sometimes maps are read-in which are created in single
    ! precision.  Values in the code may be double precision,
    ! however.  min_stomate is usually set to 1e-8, which is
    ! similar to single precision.  Depending on the maps
    ! used, this can frequently cause mass balance errors
    ! on the order of 1e-5 to 1e-8 as the old veget_cov_max
    ! and the new veget_cov_max sum to a value just less
    ! than min_stomate, which when multiplied by the pools
    ! used gives a value greater than min_stomate of carbon
    ! which is considered lost.  This first loop corrects for
    ! that and closes the mass balance.
    DO ipts=1,npts
       temp_sum_new=zero
       temp_sum_old=zero
       DO ivm=1,nvm
          temp_sum_new=temp_sum_new+veget_cov_max_new(ipts,ivm)
          temp_sum_old=temp_sum_old+veget_cov_max(ipts,ivm)
       ENDDO

       ! Debug
       IF(printlev_loc>=4 .AND. ipts == test_grid)THEN
          WRITE(numout,*) 'temp_sum_old: ',temp_sum_old
          WRITE(numout,*) 'temp_sum_new: ',temp_sum_new
       ENDIF
       !-

       difference=ABS(temp_sum_old-temp_sum_new)
       ! If the difference is too big, there is a significant difference
       ! in the nobio fraction, which has to be specially taken
       ! care of.  We let a check below deal with that. This code only
       ! deals with the single/double precision issue
       IF(difference .NE. zero .AND. difference .LT. min_stomate)THEN          
          ! Rescale veget_cov_max such that the veget_cov_max is now stored in
          ! double precision
          DO ivm=1,nvm
             veget_cov_max_new_adj(ipts,ivm)=veget_cov_max_new(ipts,ivm)*&
                  temp_sum_old/temp_sum_new
          ENDDO
       ! Nothing to change.   
       ELSE        
          DO ivm=1,nvm
             veget_cov_max_new_adj(ipts,ivm)=veget_cov_max_new(ipts,ivm)
          ENDDO
       ENDIF
    ENDDO
       
 !! 2. Initialization
    
    !! 2.1 Initialize variables
    delta_veget_cov(:,:) = zero 
    loss_gain(:,:) = zero
    veget_cov_max_group(:,:) = zero
    veget_cov_max_new_group(:,:) = zero

    ! Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)

    !! 2.2 Initialize check for mass balance closure
    IF (err_act.GT.1) THEN

       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO iele = 1,nelements

          ! Biomass pool + bm_to_litter
          DO ipar = 1,nparts
            DO icir=1, ncirc
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                    circ_class_biomass(:,:,icir,ipar,iele) * &
                    circ_class_n(:,:,icir) * veget_cov_max(:,:)
            ENDDO
            pool_start(:,:,iele) = pool_start(:,:,iele) + &
                (bm_to_litter(:,:,ipar,iele) + turnover_daily(:,:,ipar,iele)) * &
                veget_cov_max(:,:)
          ENDDO

          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO

          ! The biomass harvest pool is expressed in gC pixel-1 So, it 
          ! shouldn't be multiplied by veget_cov_max but it should be divided
          ! by pixel_area to obtain gC m-2.
          DO ipts = 1,npts
             pool_start(ipts,:,iele) = pool_start(ipts,:,iele) + &
                  SUM(harvest_pool(ipts,:,:,iele),2) / &
                  pixel_area(ipts)
          ENDDO

       ENDDO

       !Denitrifying bacteria (only C)
       pool_start(:,:,icarbon) = pool_start(:,:,icarbon) + &
               bact(:,:) * veget_cov_max(:,:)

       !Nitrogen only
       DO ispec = 1,nnspec
          pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
                  soil_n_min(:,:,ispec) * veget_cov_max(:,:)
       ENDDO
  
       ! Initialize area check and intermediate mass balance checks
       veget_cov_max_begin(:,:) = veget_cov_max(:,:)
       atm_to_bm_old(:,:,:) = atm_to_bm(:,:,:)
       pool_end(:,:,:) = zero

    ENDIF ! ld_stop .OR. ld_warn
    

!! 3. Calculate common variables 
 
    !  Remember that when age classes are being used the maps have
    !  fewer PFTs (::nvmap) than the simulation (::nvm). For several 
    !  calculations the total cover of all age class within its group 
    !  (::veget_cov_max_group) is required.  We also need the vegetation 
    !  fraction of the PFT map arranged into groups.  Notice that this 
    !  number is already given in terms of groups, but it is stored 
    !  on a grid using all nvm pfts, i.e. only nvmap values of
    !  veget_cov_max_new(ipts,:) are non-zero.
    DO ipts = 1,npts
       DO ivm=1,nvm
          igroup=agec_group(ivm)
          veget_cov_max_group(ipts,igroup) = veget_cov_max_group(ipts,igroup) + &
                        veget_cov_max(ipts,ivm)
          veget_cov_max_new_group(ipts,igroup) = veget_cov_max_new_group(ipts,igroup) + &
                        veget_cov_max_new_adj(ipts,ivm)
       ENDDO ! loop over PFTs
    ENDDO ! loop over #pixels - domain size

    ! Debug
    IF (printlev_loc>=4) THEN
       DO ipts=1,npts
          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'Finished computing veget_cov_max_group.'
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,1)
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,2)
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,5)
          ENDIF
       ENDDO
    ENDIF
    !-


 !! 4. Check consistency of the LCC maps

    ! Within each pixel the fraction of losses should equal the fraction 
    ! of gains. The soil carbon avaible from the losses is averaged and 
    ! used as the initial soil carbon for the gains. This assumption is 
    ! acceptable when the fraction of none biological land uses remains 
    ! constant. If this fraction is changing i.e. following glacier 
    ! retreat, the current approach will need to be checked and most 
    ! likely adjusted. The same applies in case vegetation is removed 
    ! for urban sprawing. This routine will need to decide whether the 
    ! soil carbon is simply buried under the hard surfaces or whether it 
    ! decomposes?

    !! 4.1 Calculate the change in vegetation cover
    delta_veget_cov(:,:) =  veget_cov_max_new_group(:,:) - veget_cov_max_group(:,:)
    ! Debug  
    IF (printlev_loc>=4) THEN
       DO ipts=1,npts
          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'Change in vegetation cover - before QC, ', &
                  SUM(delta_veget_cov(test_grid,:))
             WRITE(numout,*) 'Before normalization of delta_veget_cov'
             CALL print_lcc_error_message(npts,test_grid,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,3)
          ENDIF
       ENDDO
    ENDIF
    !-

    !! 4.2 Adjust all changes to be within bounds
    
    ! Read just the change in vegetation cover for the PFTs so that
    ! no change is less than min_vegfrac and that no change leads
    ! to PFTs values which are too high or too low.
    CALL adjust_delta_veget_cov(npts, veget_cov_max, veget_cov_max_group,&
         veget_cov_max_new_adj, veget_cov_max_new_group, loss_gain,&
         delta_veget_cov, losses)

    !Debug
    IF (printlev_loc>=4) THEN
       DO ipts=1,npts
          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'After normalization of delta_veget_cov and loss_gain'
             CALL print_lcc_error_message(npts,test_grid,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,3)
             CALL print_lcc_error_message(npts,test_grid,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)
          ENDIF
       ENDDO
    ENDIF

    ! Check whether gains equal losses. If not, it is unlikely that the C-balance
    ! will be closed.
    DO ipts = 1,npts

       ! Changes in the vegetation classes (including bare soil)
       change_bio(ipts) = SUM(delta_veget_cov(ipts,:))

       ! The nobio fractions are the leftover fractions when all the vegetation
       ! fractions are summed up.
       change_nobio(ipts)=(un-SUM(veget_cov_max_new_group(ipts,:)))-&
            (un-SUM(veget_cov_max_group(ipts,:)))

       IF ( ABS(change_bio(ipts)+change_nobio(ipts)) .GE. min_stomate) THEN

          ! Losses and gains don't cancel each other out
          WRITE(numout,*) "Error: LCC losses and gains don't cancel each &
               & other out in pixel, ",ipts             
          WRITE(numout,*) 'species change-veget_cov_max, veget_cov_max_new_group,'
          DO ivm = 1,nvm
             WRITE(numout,*) veget_cov_max(ipts,ivm), &
                  veget_cov_max_new_group(ipts,ivm)
          ENDDO
          WRITE(numout,*) 'Mismatch: ',change_bio(ipts)+change_nobio(ipts)
          WRITE(numout,*) 'change_bio: ',change_bio(ipts)
          WRITE(numout,*) 'change_nobio: ',change_nobio(ipts)
          WRITE(numout,*) 'sum of delta_veget_cov: ',SUM(delta_veget_cov(ipts,:))
          WRITE(numout,*) 'sum of veget_cov_max_new_group: ', &
               SUM(veget_cov_max_new_group(ipts,:))
          WRITE(numout,*) 'sum of veget_cov_max_new_adj: ',&
               SUM(veget_cov_max_new_adj(ipts,:))
          WRITE(numout,*) 'sum of veget_cov_max_group: ',&
               SUM(veget_cov_max_group(ipts,:))
          CALL ipslerr (2,'land_cover_main', &
               'Losses and gains do not cancel in land_cover_change.',&
               'Look for Error in out_orchidee file for details','')

       ELSE

          ! Losses and gains cancel each other out
          ! Debug
          IF (printlev_loc>=4) THEN
             WRITE(numout,*) "LCC losses and gains cancel each other out ", ipts
             WRITE(numout,*) 'OK: ',change_bio(ipts),change_nobio(ipts)
          ENDIF
          !-

       ENDIF

       ! All this accounting does not change the fact that we are only
       ! confident if there are no changes in the nobio fraction or if the
       ! changes or so small that they can be considered as a nummerical
       ! issue. If this is not the case, print a warning, since someone 
       ! should think this through more clearly.  This check should
       ! be taken care of already in slowproc_update.
       IF(ABS(change_nobio(ipts)) .GT. min_vegfrac)THEN

          WRITE(numout,*) 'LCC1 WARNING: Large change in nobio fraction in LCC!'
          WRITE(numout,*) 'ipts: ',ipts
          WRITE(numout,*) 'change_nobio: ',change_nobio(ipts)
          WRITE(numout,*) 'ivm veget_cov_max_new_adj veget_cov_max'
          DO ivm=1,nvm
             WRITE(numout,'(I7,2F12.8)') ivm,veget_cov_max_new_adj(ipts,ivm),&
                  veget_cov_max(ipts,ivm)
          ENDDO
          CALL ipslerr (2,'land_cover_main', 'Large change in nobio fraction',&
               'Look for LCC1 WARNING in out_orchidee file for details','')
       ENDIF

    ENDDO ! npts

    ! Debug
    IF (printlev_loc>=4) THEN
       WRITE(numout,*) 'veget_cov_max, veget_cov_max_new_adj, ',test_grid
       DO ipts = 1,npts
          DO ivm = 1,nvm
             WRITE(numout,*) ivm, veget_cov_max(test_grid,ivm), &
                  veget_cov_max_new_adj(test_grid,ivm)
          ENDDO
          WRITE(numout,*) 'SUM, ',SUM(veget_cov_max(test_grid,:)),&
               SUM(veget_cov_max_new_adj(test_grid,:))
          WRITE(numout,*) 'END of LCC'
          WRITE(numout,*) 'no frac bio veget_cov, ', &
               un-SUM(veget_cov_max(test_grid,:))
          WRITE(numout,*) 'no frac bio veget_cov_max_new, ',&
               un-SUM(veget_cov_max_new(test_grid,:))
          WRITE(numout,*) 'no frac bio veget_cov_max_new_adj, ',&
               un-SUM(veget_cov_max_new_adj(test_grid,:))
       ENDDO
    ENDIF 
    !-


 !! 5. Calculate in-situ land cover changes

    !  NOTE: within the model framework there is large conceptual
    !  difference between management and clear cuts on the one
    !  hand and LCC on the other hand. Within the model LCC
    !  DOES NOT change the biomass, ind, ... of the PFTs that
    !  experience a loss (because we store those in gC m-2). LCC 
    !  only changes the values of veget_cov_max for those PFTs. For
    !  this reason it was decided not to make use of lpj_kill.f90
    !  and sapiens_gap.f90 (as is done for forest management) 
    !  because this would be in conflict with the meaning of the 
    !  variable ::circ_class_kill. It is not impossible to work 
    !  around this issues but it could result in difficulties with
    !  the order of code in more complex cases were part of the PFT 
    !  experiences a surface loss and at the same time is ready to
    !  get thinned or clear cut. It appears cleaner to deal with
    !  LCC up front in a self-contained module.

    !! 5.1 Land cover losses within a pixel
    !  The losses need to be calculated first such that the soil
    !  carbon (and other elements) can be put in the soil&litter-bank. 
    !  Subsequently, the soil&litter-bank is used to initialize the 
    !  soil carbon of the new PFTs. The biomass is moved into a 
    !  harvest pool.
    !  NOTE: make sure that all variables for PFT1 are initialized!
    !  If not LCC won't properly work.
    DO ipts = 1, npts ! Loop over # pixels - domain size    

       !! 5.1.1 Initialize the soil&litter-bank
       !  NOTE Some of these variables have a dimension for ::nelements
       !  others don't. Make sure ::nelements is accounted for when 
       !  N is added to the model.
       soil_bank(:,:) = zero
       litter_bank(:,:,:) = zero
       struct_ltr_bank(:) = zero
       woody_ltr_bank(:) = zero
       fresh_ltr_bank(:,:) = zero
       fresh_som_bank(:,:) = zero
       moi_bank_season = zero
       moi_bank_month = zero
       bact_bank = zero
       soil_n_min_bank(:) = zero
       fresh_litter(:,:,:) = bm_to_litter(ipts,:,:,:)
       fresh_som(:,:,:) = turnover_daily(ipts,:,:,:)

       !! 5.5.2 Initialize debugging
       IF (printlev_loc>=4) THEN
          harvest_pool_old(ipts,:,:,:) = harvest_pool(ipts,:,:,:)
          test_mass(:,:) = zero

          ! Litter, som, bact and soil_n_min may get moved from one PFT
          ! but the pools are never mixed with other pools. If there is
          ! a mass balance problem, these pools can be easily checked 
          ! for mass conservation. This check which is only useful during
          ! debugging, is initialized here.
          DO iele = 1,nelements
             test_bank(1,iele,istart) = SUM(SUM(SUM(litter(ipts,:,:,:,iele),3),1) * &
                  veget_cov_max(ipts,:))
             test_bank(2,iele,istart) = SUM(SUM(som(ipts,:,:,iele),1) * &
                  veget_cov_max(ipts,:))
          ENDDO
          test_bank(3,icarbon,istart) = SUM(bact(ipts,:) * veget_cov_max(ipts,:))
          test_bank(3,initrogen,istart) = SUM(SUM(soil_n_min(ipts,:,:),2) * &
               veget_cov_max(ipts,:))
       ENDIF
       
       !! 5.1.3 Loss in PFT cover, update harvest and soil&litter-C-bank
       !  To calculate the relative change in cover fraction, PFT-specific 
       !  changes need to be normalized by the total change which is in 
       !  this case the total loss. Total losses includes losses in PFT1
       
       !+++CHECK+++
       ! the change in no_frac_bio could/should also be added in this 
       ! calculation. What happens to the carbon if a PFT is converted
       ! into a no_bio land use? There is already a ::c_buried variable
       ! but it is not used.
       total_losses = SUM(loss_gain(ipts,:), MASK = loss_gain(ipts,:) .LT. zero)
       !+++++++++++
       
       ! Calculate the factor to weight the lignin ratios by when merging.
       ! This depends on the amount of litter on the PFTs where vegetation
       ! is being cut.
       litter_weight_struc(:)=zero
       litter_weight_woody(:)=zero
       DO ivm = 1, nvm
          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN
             DO ilev=1,nlevs
                litter_weight_struc(ilev)=litter_weight_struc(ilev)+&
                     litter(ipts,istructural,ivm,ilev,icarbon) * &
                     ABS(loss_gain(ipts,ivm))
                litter_weight_woody(ilev)=litter_weight_woody(ilev)+&
                     litter(ipts,iwoody,ivm,ilev,icarbon) * &
                     ABS(loss_gain(ipts,ivm))
             ENDDO
          ENDIF
       ENDDO
       
       DO ivm = 1, nvm

          ! Initialize intermediate mass balance check
          test_mass_bank(:) = zero

          ! The PFT cover decreases, move wood into the harvest pool, the 
          ! stumps and branches are supposed to be exported from the site 
          ! so they go into the harvest pool as well but a separated pool
          ! was introduced for separate treatment in the product use module
          ! Leaves, fine roots and fruits are deposited on the soil&litter
          ! bank and will be attributed to the new PFT. This order of 
          ! calculations ensures that the litter ends up in the right PFT
          ! and that it is only decomposed once.         
          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN
             
             !! 5.1.2.1 Biomass losses of trees
             IF (is_tree(ivm)) THEN
                
                ! We will need the trunk diameters for all the circ classes in 
                ! order to figure out which harvest pool to put them in.
                diameters_temp(:) = &
                     wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

                ! Store the management stategy of the pixel and PFT
                ifm = forest_managed(ipts,ivm)                

                ! Now we have the stems diameters.  These are what we want to
                ! save, and we are going to classify them by PFT type, location, 
                ! and diameter so that we can compare to FAO statistics and do 
                ! some more interesting post-treatment (sapiens_product_use.f90).
                DO icir = 1,ncirc
                   DO iele = 1,nelements
                    ! We want to make sure we divide up the labile and carbohydrate
                    ! reserve pools in the same was as in other management.  We
                    ! assume that when a tree is cut down, it cannot rearrange 
                    ! these pools, and therefore the amount that is harvested
                    !  is the ratio that is stored in the harvest wood.  We assume
                    ! that the reserve pools are stored equally in all non-heart
                    ! tissue.  The harvest right now goes into three different
                    ! places: two harvest pools and litter.  We need the fractions
                    !  for the reserve pools going into each place.
                    total_nonheart_biomass = &
                        circ_class_biomass(ipts,ivm,icir,isapbelow,iele)+&
                        circ_class_biomass(ipts,ivm,icir,iroot,iele)+&
                        circ_class_biomass(ipts,ivm,icir,ileaf,iele)+&
                        circ_class_biomass(ipts,ivm,icir,ifruit,iele)+&
                        circ_class_biomass(ipts,ivm,icir,isapabove,iele)
                    ! This includes the roots and the branches, since it goes into
                    ! a different pool.
                    total_nonheart_biomass_branch=branch_ratio(ivm)*&
                        circ_class_biomass(ipts,ivm,icir,isapabove,iele)+&
                        circ_class_biomass(ipts,ivm,icir,isapbelow,iele)
                    total_nonheart_biomass_stem=(un-branch_ratio(ivm))*&
                        circ_class_biomass(ipts,ivm,icir,isapabove,iele)

                    ! These are the three ratios we need to move the reserve 
                    ! pools around. When dealing with standard land cover change 
                    ! the harvest_ratios are always defined. However, when 
                    ! dealing with a species change we try to harvest an empty 
                    ! age class, hence, the code to capture the undefined case. 
                    ! Why is the age class empty. Well in forestry we just 
                    ! harvested an older age class, in kill we replace that 
                    ! older age class by a younger age class but here we harvest 
                    ! it again before it was ever replanted.
                    IF (total_nonheart_biomass .GT. min_stomate) THEN
                      harvest_ratio_branch=total_nonheart_biomass_branch/&
                           total_nonheart_biomass
                      harvest_ratio_stem=total_nonheart_biomass_stem/&
                           total_nonheart_biomass
                      harvest_ratio_litter=un - harvest_ratio_stem - &
                           harvest_ratio_branch
                    ELSE
                      harvest_ratio_branch=zero
                      harvest_ratio_stem=zero
                      harvest_ratio_litter=un - harvest_ratio_stem - &
                           harvest_ratio_branch 
                    END IF

                    dia_loop:DO idia=1,ndia_harvest

                      ! Move the harvest to the correct diameter class and
                      ! account for the largest diameter class because
                      ! harvest_pool_bound has only ndia_harvest+1 dimensions
                      IF ( (diameters_temp(icir).LE.harvest_pool_bound(idia)).OR. &
                         ((diameters_temp(icir).GT.harvest_pool_bound(ndia_harvest)) .AND. &
                         (diameters_temp(icir).LT.harvest_pool_bound(ndia_harvest+1))) ) THEN

                         ! This is the correct diameter class
                         ! Move wood (excluding the branches) and the carbres 
                         ! and labile C contained in the wood into the harvest 
                         ! pool. Note that the unit of circ_class_biomass is gC 
                         ! tree-1, so multiply with the number of trees to obtain 
                         ! gC m-2. To avoid complexities when dealing with the 
                         ! product pool multiply with the cover change 
                         ! and the surface area of the pixel to obtain a weighted 
                         ! mean expressed in gC.
                         harvest_pool(ipts,ivm,idia,iele) = &
                              harvest_pool(ipts,ivm,idia,iele) + &  
                              ABS(loss_gain(ipts,ivm)) * pixel_area(ipts) * & 
                              ( (un - branch_ratio(ivm)) * circ_class_n(ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapabove,iele) + &
                              circ_class_biomass(ipts,ivm,icir,iheartabove,iele)) +&
                              ! reserves
                              harvest_ratio_stem * circ_class_n(ipts,ivm,icir) * &
                              (circ_class_biomass(ipts,ivm,icir,icarbres,iele) + &
                              circ_class_biomass(ipts,ivm,icir,ilabile,iele) ) )

                         ! Associated harvest variables
                         harvest_type(ipts,ivm) = ifm
                         harvest_cut(ipts,ivm) = icut_lcc_wood
                         harvest_area(ipts,ivm) = ABS(loss_gain(ipts,ivm) * &
                              pixel_area(ipts))

                         ! Move stumps and branches and the carbres and labile C 
                         ! contained in them into the harvest pool with the 
                         ! smallest dimenisons. Stumps and branches are removed 
                         ! from the site but can now be treated separately from 
                         ! stem wood in the product use module (sapiens_product_use.f90)       
                         harvest_pool(ipts,ivm,1,iele) = &
                              harvest_pool(ipts,ivm,1,iele) + &
                              ABS(loss_gain(ipts,ivm)) * pixel_area(ipts) * &
                              ! branches
                              ( branch_ratio(ivm) * circ_class_n(ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapabove,iele) + &
                              circ_class_biomass(ipts,ivm,icir,iheartabove,iele)) + &
                              ! reserves
                              harvest_ratio_branch * circ_class_n(ipts,ivm,icir) * &
                              (circ_class_biomass(ipts,ivm,icir,icarbres,iele) + &
                              circ_class_biomass(ipts,ivm,icir,ilabile,iele) )  + &
                              ! stumps (all icarbres and ilabile is distributed
                              ! over the stems and branches)
                              circ_class_n( ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapbelow,iele) + &
                              circ_class_biomass(ipts,ivm,icir,iheartbelow,iele) ) )
                         
                         !+++CHECK+++
                         ! previous values of harvest_type and harvest_cut will 
                         ! always be overwritten. Storing just a single value is 
                         ! OK but it should be the value representing the largest 
                         ! share of the biomass.
                         ! Associated harvest variables
                         harvest_type(ipts,ivm) = ifm
                         harvest_cut(ipts,ivm) = icut_lcc_res
                         harvest_area(ipts,ivm) = ABS(loss_gain(ipts,ivm) * &
                              pixel_area(ipts))
                         !+++++++++++
                        
                         ! Deposit the leaves, roots and fruit into a temporary 
                         ! litter pool and place it in the soil and litter bank
                         fresh_litter(ivm,ileaf,iele) = &
                              fresh_litter(ivm,ileaf,iele) + &
                              circ_class_biomass(ipts,ivm,icir,ileaf,iele) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,iroot,iele) = &
                              fresh_litter(ivm,iroot,iele) + &
                              circ_class_biomass(ipts,ivm,icir,iroot,iele) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,ifruit,iele) = &
                              fresh_litter(ivm,ifruit,iele) + &
                              circ_class_biomass(ipts,ivm,icir,ifruit,iele) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,icarbres,iele) = &
                              fresh_litter(ivm,icarbres,iele) + &
                              harvest_ratio_litter*&
                              circ_class_biomass(ipts,ivm,icir,icarbres,iele) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,ilabile,iele) = &
                              fresh_litter(ivm,ilabile,iele) + &
                              harvest_ratio_litter*&
                              circ_class_biomass(ipts,ivm,icir,ilabile,iele) * &
                              circ_class_n(ipts,ivm,icir)

                         EXIT dia_loop

                      ENDIF ! correct diameter class

                      ! If we are here, it means the diameter of this trunk is larger 
                      ! than the upper boundary of our harvest bins.  Considering the 
                      ! upper boundary is set to be 99999 meters in constants.f90, 
                      ! something has gone wrong!
                      IF(diameters_temp(icir) .GE. harvest_pool_bound(ndia_harvest+1))THEN

                         WRITE(numout,*) 'Our tree is too big to fit into the harvest pools.'
                         WRITE(numout,*) 'ipts, ivm, icir: ',ipts, ivm, icir
                         WRITE(numout,*) 'diameters_temp(icir), harvest_pool_bound(idia)',&
                              diameters_temp(icir),harvest_pool_bound(ndia_harvest+1)
                         CALL ipslerr (3,'land_cover_main', &
                              'harvest pool error (see output file)','','')

                      ENDIF

                    ENDDO dia_loop

                   ENDDO ! nelements

                ENDDO ! ncirc

             !! 5.1.2.2 Biomass losses of grasses and crops
             !  During LCC grasses and crops are simply plowed under 
             !  and should thus be move into ::fresh_litter which is
             !  moved later on into the litter bank ::fresh_ltr_bank
             !  and finally into ::bm_to_litter. The bank is needed
             !  to take a weighted average.
             ELSE

                DO iele = 1,nelements
                    DO icir=1,ncirc
                        fresh_litter(ivm,:,iele) = fresh_litter(ivm,:,iele) + &
                            circ_class_biomass(ipts,ivm,icir,:,iele)* &
                            circ_class_n(ipts,ivm,icir)
                    ENDDO
                ENDDO
 
             ENDIF ! is_tree(ivm)

             !! 5.1.2.3 Deposit soil and litter and soil&litter bank
             ! Deposit the released soil&litter pools on the soil&litter-bank. 
             ! The units of the C-pools are in gC m-2, hence, they need to be 
             ! weighted by their relative change in cover fraction to obtain 
             ! the C-pool (gC pixel-2) of the soil&litter-C-bank.
             ! Mass_removed (g C) = x1*X1 + x2*X2 + x3*X3 where x1, x2, and x3 
             ! are the change in veget_cov_max and X1, X2 and X3 are the 
             ! C density of a given variable (g C m-2). Mass_removed will have
             ! to be redistributed over the PFTs that increased their surface
             ! area. Because we deal with net land cover changes we don't know 
             ! the bi-directional changes and so the maxss_removed will be 
             ! averaged and this average will be redistributed. In other words,
             ! Mass_redistributed = y1*Y + y2*Y + y3*Y Where y1, y2 and y3 are 
             ! the increases in veget_max for PFT that gained cover fraction and
             ! Y is the average C-density. Mass conservation dictates that
             ! Mass_removed = Mass_distributed so Y (g C m-2) can de calculated
             ! as (x1*X1+x2*X2+x3*X3)/(y1+y2+y3). Note that y1+y2+y3 = total gain 
             ! and by definition total_gain = total_loss. This is the approach
             ! implemented below.
             litter_bank(:,:,:) = litter_bank(:,:,:) + &
                  loss_gain(ipts,ivm) * litter(ipts,:,ivm,:,:) / total_losses
             soil_bank(:,:) =  soil_bank(:,:) + &
                  loss_gain(ipts,ivm) * som(ipts,:,ivm,:) / total_losses
             fresh_ltr_bank(:,:) = fresh_ltr_bank(:,:) + &
                  loss_gain(ipts,ivm) * fresh_litter(ivm,:,:) / total_losses
             fresh_som_bank(:,:) = fresh_som_bank(:,:) + &
                  loss_gain(ipts,ivm) * fresh_som(ivm,:,:) / total_losses
             bact_bank = bact_bank + &
                  loss_gain(ipts,ivm) * bact(ipts,ivm) / total_losses
             soil_n_min_bank(:) = soil_n_min_bank(:) + &
                  loss_gain(ipts,ivm) * soil_n_min(ipts,ivm,:) / total_losses
             DO ilev=1,nlevs

                IF(litter_weight_struc(ilev) .GE. min_stomate)THEN
           
                   struct_ltr_bank(ilev) = struct_ltr_bank(ilev) + &
                        lignin_struc(ipts,ivm,ilev) * &
                        (litter(ipts,istructural,ivm,ilev,icarbon) * &
                        ABS(loss_gain(ipts,ivm))) / &
                        litter_weight_struc(ilev)
                ENDIF

                IF(litter_weight_woody(ilev) .GE. min_stomate)THEN
                   
                   woody_ltr_bank(ilev) = woody_ltr_bank(ilev) + &
                        lignin_wood(ipts,ivm,ilev) * &
                        (litter(ipts,iwoody,ivm,ilev,icarbon) * &
                        ABS(loss_gain(ipts,ivm))) / &
                        litter_weight_woody(ilev)
                ENDIF

             ENDDO

             ! In stomate_season.f90 this variable is only calculated if 
             ! the biomass(ileaf) > 0. As long as veget_cov_max is 0, this 
             ! variable does not get a value. We need a value
             ! to calculate ::KF which in turn is needed to calculate the biomass
             ! of the seedlings we will prescribe. At this point in the calculation
             ! the moiavail_growingseason from PFT1 is still zero so it is not 
             ! accounted for yet. Its exact value depends on the PFT that will
             ! be established on PFT1 and it will be accounted for later in the 
             ! code
             IF (ivm .GT. 1) THEN

                moi_bank_season = moi_bank_season + loss_gain(ipts,ivm) / &
                     total_losses * ( un - wstress_season(ipts,ivm))
                moi_bank_month = moi_bank_month + loss_gain(ipts,ivm) / &
                     total_losses * (un - wstress_month(ipts,ivm))

             ENDIF

             ! Intermediate error checking for losses only
             IF (printlev_loc>=4) THEN

                ! We are only interested in the PFTs that lost part 
                ! of their share in the pixel, So we use a temporary
                ! variable for pool_start. To keep things simple
                ! the same is done for pool_end
                test_mass(:,istart) = test_mass(:,istart) + &
                     pool_start(ipts,ivm,:)

                !! 6.1 Account for the carbon & nitrogen left on-site
                !  The amount of carbon/nitrogen per tree or per m-2 has
                !  not changed but the surface area covered by the PFT
                !  has decreased. Don use the PFT dimension in pool_end
                !  because the banks no longer have PFT dimenions either
                DO iele = 1,nelements

                   ! Biomass pool
                   DO icir = 1,ncirc
                      test_mass(iele,iend) = test_mass(iele,iend) + &
                           SUM(circ_class_biomass(ipts,ivm,icir,:,iele))*&
                           circ_class_n(ipts,ivm,icir) * &
                           (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))
                   ENDDO

                   ! bm_to_litter and turnover_daily
                   test_mass(iele,iend) = test_mass(iele,iend) + &
                        SUM(bm_to_litter(ipts,ivm,:,iele) + &
                        turnover_daily(ipts,ivm,:,iele)) * &
                        (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))

                   ! Litter pool (gC m-2) *  (m2 m-2) 
                   test_mass(iele,iend) = test_mass(iele,iend) + &
                        SUM(SUM(litter(ipts,:,ivm,:,iele),2)) * &
                        (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))

                   ! Soil carbon (gC m-2) *  (m2 m-2)
                   test_mass(iele,iend) = test_mass(iele,iend) + &
                        SUM(som(ipts,:,ivm,iele)) * &
                        (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))

                ENDDO !nelements

                !Denitrifying bacteria (only C)
                test_mass(icarbon,iend) = test_mass(icarbon,iend) + &
                     bact(ipts,ivm) * &
                     (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))

                ! Nitrogen only
                test_mass(initrogen,iend) = test_mass(initrogen,iend) + &
                     SUM(soil_n_min(ipts,ivm,:)) * & 
                     (veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm))
                
                !! 6.2 Account for the pools where the C and N was moved to
                !  When the total surface area of a PFT was decreased, the 
                !  carbon and nitrogen that was removed was moved into several
                !  so called "banks" and the harvest pool. 
                DO iele = 1,nelements

                   ! The biomass harvest pool is expressed in gC pixel-1 So, it 
                   ! shouldn't be multiplied by veget_cov_max but it should be 
                   ! divided by pixel_area to obtain gC m-2.
                   test_mass(iele,iend) = test_mass(iele,iend) + &
                        SUM(harvest_pool(ipts,ivm,:,iele) - &
                        harvest_pool_old(ipts,ivm,:,iele)) / &
                        pixel_area(ipts)
                   test_mass_bank(iele) = test_mass_bank(iele) - &
                        SUM(fresh_ltr_bank(:,iele) + fresh_som_bank(:,iele)) * &
                        total_losses
                   test_mass_bank(iele) = test_mass_bank(iele) - &
                        SUM(soil_bank(:,iele)) * total_losses
                   test_mass_bank(iele) = test_mass_bank(iele) - &
                        SUM(SUM(litter_bank(:,:,iele),2)) * &
                        total_losses

                ENDDO

                ! Pool with only carbon
                test_mass_bank(icarbon) = test_mass_bank(icarbon) - &
                     bact_bank * total_losses

                ! Pools with only nitrogen
                test_mass_bank(initrogen) = test_mass_bank(initrogen) - &
                     SUM(soil_n_min_bank(:)) * total_losses
                
                DO iele = 1,nelements
                   IF (ABS(test_mass(iele,iend)-test_mass(iele,istart) + &
                        test_mass_bank(iele)).GT.min_stomate) THEN
                      WRITE(numout,*) 'ivm, iele, test_mass(istart), test_mass(iend), &
                           & test_mass_bank, diff'
                      WRITE(numout,'(2I3,4F20.10)') ivm, iele, &
                           test_mass(iele,istart), test_mass(iele,iend), &
                           test_mass_bank(iele), test_mass(iele,iend) - &
                           test_mass(iele,istart) + test_mass_bank(iele)
                      CALL ipslerr(3,'sapiens_lcchange','When accounting &
                           & for the C and N due to decreases in PFTs', &
                           'the C/N is not conserved','')
                   ELSE
                      ! Debug
                      IF (printlev_loc>=4) THEN
                         WRITE(numout,'(A,2I3)') 'OK intermediate mass balance &
                              & check',ivm,iele
                      ENDIF
                      !-
                   ENDIF ! diff is zero
                ENDDO ! nelements

             ENDIF !printlev_loc

          ENDIF ! IF (loss_gain(ivm,ipts) < -min_stomate)

       ENDDO ! Loop over # PFT's

       !! 5.2 Land cover gains within a pixel
       !  The consecutive loops over nvm cannot be merged because we first
       !  need to deposit all the available pools on the bank before we
       !  can withdraw the deposits to initialize new PFTs with the weighted
       !  mean of the deposits

       !! 5.2.1 There is no land cover change or there is a loss      
       DO ivm = 1,nvm

          ! Debug
          IF (printlev_loc>=4) THEN
             ! Initialize intermediate mass conservation checks
             test_mass(:,istart) = zero
          ENDIF
          !-

          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN
             
             ! There is no land cover change or there is a loss. In case there 
             ! is no land cover change nothing should be done. The case in which 
             ! there is a loss is already dealt with above - see comment 
             ! below §5.2 to learn why the case of a loss is not integrated in 
             ! this IF-statement. Basically we need to put all the lost C/N on 
             ! the bank first before we can redistribute it. 
 
          ! Increase in the extent of a PFT
          ELSEIF ( loss_gain(ipts,ivm) .GE. min_stomate ) THEN

             !! 5.2.1 Check wether this is the youngest age class
             !  If the redistribution of the LCC map in age classes is correctly 
             !  implemented a new age class should always be established in the 
             !  youngest age class which should be the first of the group. This 
             !  implies that in this case ::ivm is the first of the group and 
             !  ivm + nagec - 1 is the last age class of the group
             IF (is_tree(ivm)) THEN

                igroup = INT(agec_group(ivm))
                nagec_temp = nagec_pft(igroup)

                IF (agec_group(ivm+nagec_temp-1) .NE. igroup) THEN
                   
                   ! ::ivm and ivm+nagec-1 are in a different ::igroup 
                   ! apparently ::ivm is not the youngest age class
                   WRITE(numout,*) 'Error: problem with the redistribution of the LCC maps'
                   WRITE(numout,*) 'ipts, ivm, nagec_temp:  ',ipts, ivm
                   WRITE(numout,*) 'veget_cov_max, loss_gain: ',veget_cov_max(ipts,ivm),&
                        loss_gain(ipts,ivm)
                   WRITE(numout,*) 'nagec_temp, igroup:  ',nagec_temp,igroup
                   WRITE(numout,*) 'agec_group(ivm), agec_group(ivm+nagec_temp-1):  ',&
                        agec_group(ivm), agec_group(ivm+nagec_temp-1)
                   CALL ipslerr (3,'land_cover_main','Cannot find proper age classes!','','')

                ENDIF

             ENDIF

             !! 5.2.2 The new PFT does not exist yet
             IF (veget_cov_max(ipts,ivm) .LT. min_stomate) THEN

                !! 5.2.2.1 Initialize new and dummy variables
                new_circ_class_biomass(:,:,:,:,:) = zero
                new_circ_class_n(:,:,:) = zero
                new_lm_lastyearmax(:,:) = zero
                new_age(:,:) = zero
                new_leaf_frac(:,:,:) = zero
                new_atm_to_bm(:,:,:) = zero
                new_everywhere(:,:) = zero 
                dum_when_growthinit(:,:) = large_value
                dum_npp_longterm(:,:) = zero
                dum_PFTpresent(:,:) = .TRUE.
                plant_status(ipts,ivm) = iprescribe
               
                !! 5.2.2.2 Assign value to moiavail_growingseason
                !  Remember to add the moisture from PFT1 that will be present 
                !  if PFT1 is planted with PFT ivm. Note ::loss_gain is here for
                !  PFT1 but ::vir_moiavail_growingseason is for PFT ivm.
                !moiavail_growingseason(ipts,ivm) = moi_bank + &
                !     vir_moiavail_growingseason(ipts,ivm) * &
                !     loss_gain(ipts,1) / total_losses
                ! k_latosa_adapt is adjusted every day. wstress is no longer
                ! used in prescribe
                !wstress_season(ipts,ivm) = moi_bank + &
                !     vir_wstress_fac(ipts,ivm) * &
                !     loss_gain(ipts,1) / total_losses 

                !! 5.2.2.3 Prescribe values for newly established PFTs
                ! Initialize density of individuals, biomass, allocation factors,
                ! leaf age distribution, ... to some reasonable value
                ! ::veget_cov_max is not updated yet, therefore loss_gain is passed in 
                ! the argument list. As ::veget_cov_max is used in the IF-THEN-statements
                ! we won't update it until the end of this routine. If we would
                ! update before that, the same PFT may be subject to conflicting actions
                ! i.e. prescribe a new PFT and then merge it with the other age-classes
                ! in the PFT-group.
                CALL prescribe(npts, loss_gain, dt_days, dum_PFTpresent, &
                     new_everywhere, dum_when_growthinit, new_leaf_frac, circ_class_dist, &
                     new_circ_class_n, new_circ_class_biomass, new_atm_to_bm, &
                     forest_managed, KF, &
                     plant_status, new_age, dum_npp_longterm, new_lm_lastyearmax, &
                     tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt, &
                     light_tran_to_level_season, EndOfYear,lpft_replant, species_change_map)

                circ_class_biomass(ipts,ivm,:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)
                circ_class_n(ipts,ivm,:) = new_circ_class_n(ipts,ivm,:)
                lm_lastyearmax(ipts,ivm) = new_lm_lastyearmax(ipts,ivm)
                age(ipts,ivm) = new_age(ipts,ivm)
                leaf_frac(ipts,ivm,:) = new_leaf_frac(ipts,ivm,:)
                atm_to_bm(ipts,ivm,:) = new_atm_to_bm(ipts,ivm,:)
                everywhere(ipts,ivm) = new_everywhere(ipts,ivm)
                PFTpresent(ipts,ivm) = .TRUE.

                ! Update a couple forestry parameters.  I initialize these to
                ! one instead of zero since forestry is called after this
                ! routine, and it looks at last_cut.  This is not a real
                ! problem since the trees we prescribe generally are the height of
                ! saplings and not seeds.
                age_stand(ipts,ivm) = 1
                last_cut(ipts,ivm) = 1

                ! Initialize the soil and litter carbon pools. When PFTs were removed
                ! their soil and litter carbon was deposited on the bank. It is assumed
                ! that the newly established PFT are established uniformly on all the
                ! removed PFTs. Hence there is no preferential LCC sequence. For example
                ! we deforest a needle and broadleaved PFT and replace it by a crop and
                ! a grassland the crop and grassland are both established on the former
                ! needle and broadleaved PFT rather than the crop being entirely on the 
                ! former needleleaved PFT and grassland on the former broadleaved PFT.
                ! Further, the LCC-maps should guarantee that the losses equal the gains
                ! which ensures mass preservation. For these reasons we can simply use the
                ! mean soil and litter carbon on the bank (gC m-2) to initialize the soil
                ! and litter crabon of the newly established PFT's. 
                litter(ipts,:,ivm,:,:) = litter_bank(:,:,:)
                som(ipts,:,ivm,:) =  soil_bank(:,:)
                lignin_struc(ipts,ivm,:) = struct_ltr_bank(:)
                lignin_wood(ipts,ivm,:) = woody_ltr_bank(:)
                bm_to_litter(ipts,ivm,:,:) = fresh_ltr_bank(:,:)
                turnover_daily(ipts,ivm,:,:) = fresh_som_bank(:,:)
                soil_n_min(ipts,ivm,:) = soil_n_min_bank(:)
                bact(ipts,ivm) = bact_bank


             !! 5.2.3 The new PFT does already exist 
             ELSEIF (veget_cov_max(ipts,ivm) .GE. min_stomate) THEN

                !  We want to make use of prescribe_prognostic but it should be noted
                !  that this PFT already contains biomass. Therefore we will create a
                !  temporary PFT without biomass which can be used to establish the
                !  new vegetation within this PFT. For most of the INTENT(inout)
                !  variables of prescribe_prognostic we receive temporary variables 
                !  back this helps us to calculate the weighted mean of the newly 
                !  established vegetation and the vegetation that is already there.
                !  For some other variables we receive dummies as we are simply not
                !  going to use these values but will continue using the values of the
                !  vegetation that is already there.

                !! 5.2.3.1 Initialize new and dummy variables
                new_circ_class_biomass(:,:,:,:,:) = zero
                new_circ_class_n(:,:,:) = zero
                new_lm_lastyearmax(:,:) = zero
                new_age(:,:) = zero
                new_leaf_frac(:,:,:) = zero
                new_atm_to_bm(:,:,:) = zero
                new_everywhere(:,:) = zero 
                dum_when_growthinit(:,:) = large_value
                dum_npp_longterm(:,:) = zero
                dum_PFTpresent(:,:) = .TRUE.
                plant_status(ipts,ivm) = iprescribe
                
                !! 5.2.3.2 Merge the new and already available vegetation
                !  Veget_max after PFT expansion. As ::veget_cov_max is used in the 
                !  IF-statements we won't update it until the end of this routine. If 
                !  we would updatebefore that, the same PFT may be subject to 
                !  conflicting actions.
                share_expanded = veget_cov_max(ipts,ivm) / &
                     ( veget_cov_max(ipts,ivm) + loss_gain(ipts,ivm) )

                ! We also need a scaling factor which includes the litter. This is for growing
                ! a new PFT on the soil mixture left from all the land we cleared.
                DO ilev=1,nlevs

                   DO ilit=1,nlitt

                      IF(litter(ipts,ilit,ivm,ilev,icarbon) .GE. min_stomate)THEN
                        
                         litter_weight_expanded(ilit,ilev) = &
                              litter(ipts,ilit,ivm,ilev,icarbon) * &
                              veget_cov_max(ipts,ivm)/ &
                              (litter(ipts,ilit,ivm,ilev,icarbon) * veget_cov_max(ipts,ivm) + &
                              litter_bank(ilit,ilev,icarbon) * loss_gain(ipts,ivm))

                      ELSE
                         
                         litter_weight_expanded(ilit,ilev)=zero

                      ENDIF

                   END DO

                ENDDO

                ! Remember to add the moisture from PFT1 that will be present 
                ! if PFT1 is planted with PFT ivm. Note ::loss_gain is here for
                ! PFT1 but ::vir_wstress_fac is for PFT ivm.
                ! Here we need to be careful.  loss_gain(ipts,ivm) will always be
                ! negative, but we are using loss_gain(ipts,1) which may be negative 
                ! or positive. So we have to take absolute values.
                IF(ABS(total_losses) .GT. min_stomate)THEN

                   wstress_season(ipts,ivm) = un - ( (un - wstress_season(ipts,ivm)) * &
                        share_expanded + (moi_bank_season  * &
                        ABS(loss_gain(ipts,1)) / ABS(total_losses) ) * (un - share_expanded))
                   wstress_month(ipts,ivm) = un - ( (un - wstress_month(ipts,ivm)) * &
                        share_expanded + (moi_bank_month  * &
                        ABS(loss_gain(ipts,1)) / ABS(total_losses) ) * (un - share_expanded))

                ! In this case, we expect the nobio fraction to change and we're okay
                ! with it.  So just leave the long-term water stresses at their
                ! previous values.
                ELSEIF(err_act .EQ. 2)THEN

                   WRITE(numout,*) 'WARNING: ipts,ivm: ',ipts,ivm
                   WRITE(numout,*) 'total_losses: ',total_losses
                   CALL ipslerr (2,'land_cover_main', &
                        'We have no losses on this pixel, which causes a divide by zero, but ',&
                        'you have indicated that you understand the situation so we continue.',&
                        'Look for Error in out_orchidee file for details')

                ! We are not okay with this situation.  Stop with an error.
                ELSEIF(err_act .EQ. 3)THEN
                  
                   WRITE(numout,*) 'ERROR: ipts,ivm: ',ipts,ivm
                   WRITE(numout,*) 'total_losses: ',total_losses
                   CALL ipslerr (3,'land_cover_main', &
                        'We have no losses on this pixel, which causes a divide by zero, and ',&
                        'you have not indicated that you understand the situation.',&
                        'Look for Error in out_orchidee file for details')
                   
                ENDIF

                !! 5.2.3.3 Initialize the newly established vegetation: 
                !  Initialize the newly established vegetation: density of 
                !  individuals, biomass, allocation factors, leaf age distribution, 
                !  etc to some reasonable value ::veget_cov_max is not updated yet, 
                !  therefore loss_gain is passed in the argument list.
                CALL prescribe (npts, loss_gain, dt_days, dum_PFTpresent, &
                     new_everywhere, dum_when_growthinit, new_leaf_frac, circ_class_dist, &
                     new_circ_class_n, new_circ_class_biomass, new_atm_to_bm, &
                     forest_managed, KF, &
                     plant_status, new_age, dum_npp_longterm, new_lm_lastyearmax, &
                     tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt, &
                     light_tran_to_level_season, EndOfYear, lpft_replant, species_change_map)

                !! 5.2.3.3.1 Merge biomass of new and already available trees
                IF (is_tree(ivm)) THEN

                   ! Copy the number of individuals and the biomass of the established 
                   ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                   ! and ::circ_class_biomass will be overwritten with the characteristics
                   ! of the merged vegetation
                   est_circ_class_n(:) = circ_class_n(ipts,ivm,:)
                   est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,ivm,:,:,:)
                   tmp_circ_class_n(:) = new_circ_class_n(ipts,ivm,:)
                   tmp_circ_class_biomass(:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)

                   ! Store circumference (m) in a temporary variable that can be 
                   ! sorted from small to large - note that the dimension of these
                   ! variables are 2*ncirc
                   est_circ_class_dia(:) = wood_to_dia(est_circ_class_biomass(:,:,icarbon),ivm)
                   tmp_circ_class_dia(:) = wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),ivm)

                   ! Merge the biomass        
                   CALL merge_biomass_pfts(npts, share_expanded, circ_class_n, &
                        est_circ_class_n, tmp_circ_class_n, circ_class_biomass, &
                        est_circ_class_biomass, &
                        tmp_circ_class_biomass, circ_class_dist, &
                        ipts, ivm, est_circ_class_dia, tmp_circ_class_dia)

                !! 5.2.3.3.2 Merge biomass of new and already available grass or cropland
                ELSE
                                  
                   ! Dilute the existing biomass pools and number of individuals
                   circ_class_biomass(ipts,ivm,1,:,:) = share_expanded * &
                        circ_class_biomass(ipts,ivm,1,:,:) + &
                        (un - share_expanded) * new_circ_class_biomass(ipts,ivm,1,:,:)
                   circ_class_n(ipts,ivm,1) = share_expanded * circ_class_n(ipts,ivm,1)+ &
                        (un - share_expanded) * new_circ_class_n(ipts,ivm,1)

                ENDIF
               
                !! 5.2.3.4 Calculate the PFT characteristics of the merged PFT
                !  Take the weighted mean of the existing vegetation and the new 
                !  vegetation in this PFT. Note that co2_to_bm is in gC. m-2 dt-1 
                !  so we should also take the weighted mean (rather than sum if
                !  this where absolute values).
                lm_lastyearmax(ipts,ivm) = share_expanded * lm_lastyearmax(ipts,ivm) + &
                     (un - share_expanded) * new_lm_lastyearmax(ipts,ivm)
                age(ipts,ivm) = share_expanded * age(ipts,ivm) + &
                     (un - share_expanded) * new_age(ipts,ivm)
                leaf_frac(ipts,ivm,:) = share_expanded * leaf_frac(ipts,ivm,:) + &
                     (un - share_expanded) * new_leaf_frac(ipts,ivm,:)
                atm_to_bm(ipts,ivm,:) = share_expanded * atm_to_bm(ipts,ivm,:) + &
                     (un - share_expanded) * new_atm_to_bm(ipts,ivm,:)

                ! Everywhere deals with the migration of vegetation. Copy the
                ! status of the most migrated vegetation for the whole PFT
                everywhere(ipts,ivm) = MAX(everywhere(ipts,ivm), new_everywhere(ipts,ivm))

                ! The new soil&litter pools are the weighted mean of the newly 
                ! established vegetation for that PFT (as deposited on the bank) 
                ! and the vegetation that already exists in that PFT.
                ! NOTE that in case veget_cov_max of PFT1 (bare soil) is increasing
                ! we will also move litter to the bare soil. stomate_litter.f90
                ! was adjusted such that decomposition also happens in PFT
                litter(ipts,:,ivm,:,:) = share_expanded * litter(ipts,:,ivm,:,:) + &
                        (un - share_expanded) * litter_bank(:,:,:)
                som(ipts,:,ivm,:) =  share_expanded * som(ipts,:,ivm,:) + &
                     (un - share_expanded) * soil_bank(:,:)
                lignin_struc(ipts,ivm,:) = litter_weight_expanded(istructural,:) * &
                     lignin_struc(ipts,ivm,:) + &
                     (un - litter_weight_expanded(istructural,:)) * struct_ltr_bank(:)
                lignin_wood(ipts,ivm,:) = litter_weight_expanded(iwoody,:) * &
                     lignin_wood(ipts,ivm,:) + &
                     (un - litter_weight_expanded(iwoody,:)) * woody_ltr_bank(:)
                bm_to_litter(ipts,ivm,:,:) = share_expanded * bm_to_litter(ipts,ivm,:,:) + & 
                     (un - share_expanded) * fresh_ltr_bank(:,:)
                turnover_daily(ipts,ivm,:,:) = share_expanded * turnover_daily(ipts,ivm,:,:) + &
                     (un - share_expanded) * fresh_som_bank(:,:)
                soil_n_min(ipts,ivm,:) = share_expanded * soil_n_min(ipts,ivm,:) + & 
                     (un - share_expanded) * soil_n_min_bank(:)
                bact(ipts,ivm) = share_expanded * bact(ipts,ivm) + &
                     (un - share_expanded) * bact_bank

                ! These next two variables are integers, so we use CEILING to make sure
                ! they stay integers.  FLOOR or INT would also be fine.  The use of
                ! CEILING is arbitrary.
                age_stand(ipts,ivm)  = CEILING(share_expanded * age_stand(ipts,ivm) + &
                     (un - share_expanded) * 1)
                last_cut(ipts,ivm)  = CEILING(share_expanded * last_cut(ipts,ivm) + &
                     (un - share_expanded) * 1)

             ELSE
               
                WRITE(numout,*) 'Error 1: logic of the IF-statements in land_cover_change'
   
             ENDIF ! New PFT

          ELSEIF(ABS(loss_gain(ipts,ivm)) .LE. 1e-15)THEN

             ! Do nothing.  This is okay, as far as I can tell.
             ! This seems to be the double precision limit.

          ELSE

             WRITE(numout,*) 'ipts, ivm: ',ipts,ivm
             WRITE(numout,*) 'loss_gain: ',loss_gain(ipts,ivm)
             CALL ipslerr(3, 'Logic of IF-statement in land_cover_change',&
                  'Second IF-statement where this could happen','','')

          ENDIF ! Land cover change?

       ENDDO  ! Loop over # PFT's

       ! Debug
       IF (printlev_loc>=4) THEN

          ! This part of an additional mass balance check for variables
          ! that pass through the bank. Litter, som, bact and soil_n_min
          ! may be moved from one PFT to another but they are not mixed.
          ! For each of the variables their value at the start of this
          ! subroutine should equal their value at the end
          DO iele = 1,nelements
             test_bank(1,iele,iend) = SUM(SUM(SUM(litter(ipts,:,:,:,iele),3),1) * &
                  (veget_cov_max(ipts,:)+loss_gain(ipts,:)))
             test_bank(2,iele,iend) = SUM(SUM(som(ipts,:,:,iele),1) * &
                  (veget_cov_max(ipts,:)+loss_gain(ipts,:)))
          ENDDO
          test_bank(3,icarbon,iend) = SUM(bact(ipts,:) * &
                  (veget_cov_max(ipts,:)+loss_gain(ipts,:)))
          test_bank(3,initrogen,iend) = SUM(SUM(soil_n_min(ipts,:,:),2) * &
                  (veget_cov_max(ipts,:)+loss_gain(ipts,:)))

          ! Test mass conservation
          DO iele = 1,nelements

             IF (ABS(test_bank(1,iele,iend)-test_bank(1,iele,istart)).GT.min_stomate) THEN
                WRITE(numout,*) 'iele, test_bank(istart), test_bank(iend), diff'
                WRITE(numout,'(I3,3F20.10)'),iele,test_bank(1,iele,iend), test_bank(1,iele,istart),&
                     test_bank(1,iele,iend)-test_bank(1,iele,istart)
                CALL ipslerr(2,'sapiens_lcchange','Mass balance error for litter','','')
             ELSE
                WRITE(numout,'(A,I3)') 'OK intermediate mass balance check for litter',iele
             ENDIF
             IF (ABS(test_bank(2,iele,iend)-test_bank(2,iele,istart)).GT.min_stomate) THEN
                WRITE(numout,*) 'iele, test_bank(istart), test_bank(iend), diff'
                WRITE(numout,'(I3,3F20.10)'),iele,test_bank(2,iele,iend), test_bank(2,iele,istart),&
                     test_bank(2,iele,iend)-test_bank(2,iele,istart)
                CALL ipslerr(2,'sapiens_lcchange','Mass balance error for som','','')
             ELSE
                WRITE(numout,'(A,I3)') 'OK intermediate mass balance check for som',iele
             ENDIF

             IF (ABS(test_bank(3,iele,iend)-test_bank(3,iele,istart)).GT.min_stomate) THEN
                WRITE(numout,*) 'iele, test_bank(istart), test_bank(iend), diff'
                WRITE(numout,'(I3,3F20.10)'),iele,test_bank(3,iele,iend), test_bank(3,iele,istart),&
                     test_bank(3,iele,iend)-test_bank(3,iele,istart)
                IF (iele.EQ.1) CALL ipslerr(2,'sapiens_lcchange','Mass balance error for bact','','')
                IF (iele.EQ.2) CALL ipslerr(2,'sapiens_lcchange','Mass balance error for soil_n_min','','')
             ELSE
                IF (iele.EQ.1) WRITE(numout,*) 'OK intermediate mass balance check for bact'
                IF (iele.EQ.2) WRITE(numout,*) 'OK intermediate mass balance check for soil_n_min'
             
             ENDIF ! test diff

          ENDDO ! iele

       ENDIF ! prinlev_loc
       !-

    ENDDO ! Loop over npts
    
 !! 6. Calculate new veget_cov_max values and update variables if PFT was removed 
       
    veget_cov_max(:,:) = veget_cov_max(:,:) + loss_gain(:,:)

    ! Error checking
    IF (SUM(veget_cov_max - veget_cov_max_new_adj).GT.min_stomate) THEN
       CALL ipslerr(3,'Error after updating veget_cov_max','','','')
    ENDIF

    ! Update the variables if veget_cov_max is below threshold
    ! +++CHECK+++
    ! This looks like something that will result in mass balance problems
    DO ipts = 1,npts
       
       DO ivm = 1,nvm

          IF (veget_cov_max(ipts,ivm) .LT. min_vegfrac) THEN

             !Debug
!!$             IF(printlev_loc>=4)THEN
                IF (veget_cov_max(ipts,ivm) .GT. zero) THEN
                   WRITE(numout,*) 'Cleaning PFTs below threshold', ipts, ivm
                   WRITE(numout,*) 'Vegetation lost',veget_cov_max(ipts,ivm)
                ENDIF
!!$             ENDIF
             !

             veget_cov_max(ipts,ivm) = zero
             lm_lastyearmax(ipts,ivm) = zero
             age(ipts,ivm) = zero
             leaf_frac(ipts,ivm,:) = zero
             atm_to_bm(ipts,ivm,:) = zero
             everywhere(ipts,ivm) = zero
             litter(ipts,:,ivm,:,:) = zero
             som(ipts,:,ivm,:) = zero
             lignin_struc(ipts,ivm,:) = zero
             lignin_wood(ipts,ivm,:) = zero
             bm_to_litter(ipts,ivm,:,:) = zero
             turnover_daily(ipts,ivm,:,:) = zero
             soil_n_min(ipts,ivm,:) = zero
             bact(ipts,ivm) = zero
             PFTpresent(ipts,ivm) = .FALSE.
             circ_class_biomass(ipts,ivm,:,:,:) = zero
             circ_class_n(ipts,ivm,:) = zero
             age_stand(ipts,ivm)=0
             last_cut(ipts,ivm)=0
             plant_status(ipts,ivm)= 1

          ENDIF

       ENDDO

    ENDDO
    !+++++++++++

    IF (printlev_loc>=4) THEN
       DO ipts=1,npts
 !          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'Final values before mass balance'
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new_adj,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)
 !          ENDIF
       ENDDO
    ENDIF

    ! Debug
    IF (printlev_loc>=4) THEN
       ! We don want to see C/N values for PFTs without veget_cov_max. That
       ! would indicate a problem with reading and/or processing the land
       ! cover change maps.
       WRITE(numout,*) 'Checking whether biomass and veget_cov_max are aligned'
       DO ipts = 1,npts
          WRITE(numout,*) 'ivm, veget_max, biomass'
          DO ivm = 1,nvm
             WRITE(numout,'(I2,2F20.10)') ivm, veget_cov_max(ipts,ivm),&
                  SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2))
          ENDDO
          WRITE(numout,*) 'ivm, veget_max, bm_to_litter'  
          DO ivm = 1,nvm
             WRITE(numout,'(I2,2F20.10)') ivm, veget_cov_max(ipts,ivm),&
                  SUM(bm_to_litter(ipts,ivm,:,icarbon))
          ENDDO
          WRITE(numout,*) 'ivm, veget_max, turnover_daily'
          DO ivm = 1,nvm
             WRITE(numout,'(I2,2F20.10)') ivm, veget_cov_max(ipts,ivm),&
                  SUM(turnover_daily(ipts,ivm,:,icarbon))
          ENDDO
          WRITE(numout,*) 'ivm, veget_max, litter'
          DO ivm = 1,nvm
             WRITE(numout,'(I2,2F20.10)') ivm, veget_cov_max(ipts,ivm),&
                  SUM(SUM(litter(ipts,:,ivm,:,icarbon),2))
          ENDDO
          WRITE(numout,*) 'ivm, veget_max, som'
          DO ivm = 1,nvm
             WRITE(numout,'(I2,2F20.10)') ivm, veget_cov_max(ipts,ivm),&
                  SUM(som(ipts,:,ivm,icarbon))
          ENDDO
       ENDDO
    ENDIF
    !-

!! 7. Mass balance closure
   
    IF (err_act.GT.1) THEN

       ! 7.1 Check surface area
       CALL check_surface_area("sapiens_lccchange", npts, veget_cov_max_begin, &
            veget_cov_max,'pixel')

       !! 7.2 Calculate components of the mass balance
       pool_end(:,:,:) = zero
       DO iele = 1,nelements

          ! Biomass pool + bm_to_litter
          DO ipar = 1,nparts
            DO icir=1, ncirc
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                    circ_class_biomass(:,:,icir,ipar,iele)*&
                    circ_class_n(:,:,icir) *veget_cov_max(:,:)
            ENDDO
            pool_end(:,:,iele) = pool_end(:,:,iele) + &
                (bm_to_litter(:,:,ipar,iele) + turnover_daily(:,:,ipar,iele))* veget_cov_max(:,:)
          ENDDO

          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO

          ! The biomass harvest pool is expressed in gC pixel-1 So, it 
          ! shouldn't be multiplied by veget_cov_max but it should be divided
          ! by pixel_area to obtain gC m-2. The difference between 
          ! pool_start and pool_end was accumulated over dt, therefore, 
          ! the units of ipoolchange are gC m-2 dt-1.
          DO ipts = 1,npts
             pool_end(ipts,:,iele) = pool_end(ipts,:,iele) + &
                  SUM(harvest_pool(ipts,:,:,iele),2) / &
                  pixel_area(ipts)
          ENDDO

       ENDDO

       !Denitrifying bacteria (only C)
       pool_end(:,:,icarbon) = pool_end(:,:,icarbon) + &
               bact(:,:) * veget_cov_max(:,:)
 
       ! Nitrogen only
       DO ispec = 1,nnspec
          pool_end(:,:,initrogen) = pool_end(:,:,initrogen) + &
              soil_n_min(:,:,ispec) * veget_cov_max(:,:)
       ENDDO

       !! 7.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,:,iatm2land,iele) = &
               check_intern(:,:,iatm2land,iele) + &
               atm_to_bm(:,:,iele) * veget_cov_max(:,:) * dt_days
          check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
               pool_start(:,:,iele)) 
       ENDDO

       closure_intern(:,:,:) = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
            DO ivm=1, nvm
             ! Debug
             IF (printlev_loc>=4) THEN              
                IF(iele .EQ. 1) WRITE(numout,"(9999(G12.5,:,','))") &
                    'imbc, ivm, check_intern', test_grid , imbc, &
                    ivm, check_intern(test_grid,ivm,imbc,iele)
             ENDIF
             !-
            ENDDO
             closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
                  check_intern(:,:,imbc,iele)
          ENDDO
       ENDDO

       ! 7.3 Check mass balance closure
       CALL check_mass_balance("sapiens_lccchange", closure_intern, npts, &
            pool_end, pool_start, veget_cov_max, 'pixel')
    
    ENDIF ! err_act.GT.1    

    !Debug
    ! Check to see if the nobio fraction has changed
    DO ipts=1,npts
       frac_nobio_end(ipts,1)=un-SUM(veget_cov_max(ipts,:))
       IF(abs(frac_nobio_start(ipts,1) - frac_nobio_end(ipts,1)) .GT. min_stomate)THEN
          IF(hack_lcc)THEN
             WRITE(numout,*) 'ERROR: nobio fraction changed during LCC!'
             WRITE(numout,*) 'ipts, frac_nobio_start(ipts,1): ',ipts, frac_nobio_start(ipts,1)
             WRITE(numout,*) 'ipts, frac_nobio_end(ipts,1): ',ipts, frac_nobio_end(ipts,1)
             CALL ipslerr (3,'land_cover_main', 'nbio fraction error.',&
                  'Check output file for details.','')
          ELSE
             WRITE(numout,*) 'WARNING: nobio fraction changed during LCC!'
             WRITE(numout,*) 'ipts, frac_nobio_start(ipts,1): ',ipts, frac_nobio_start(ipts,1)
             WRITE(numout,*) 'ipts, frac_nobio_end(ipts,1): ',ipts, frac_nobio_end(ipts,1)
             CALL ipslerr (2,'land_cover_main', 'nbio fraction error.',&
                  'Check output file for details.','')
          ENDIF

       ENDIF
    ENDDO
    !

    IF (printlev.GE.4) WRITE(numout,*) 'Leaving land_cover_change_main'

  END SUBROUTINE land_cover_change_main

! ================================================================================================================================
!! SUBROUTINE   : age_class_distr
!!
!>\BRIEF        Redistribute biomass, litter, soilcarbon and water across
!!              the age classes
!!
!! DESCRIPTION  : Following growth, the trees from an age class may have become
!! too big to belong to this age class. The biomass, litter, soilcarbon and
!! soil water then need to be moved from one age class to the next age class.
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE age_class_distr(npts, circ_class_n, circ_class_biomass, &
       veget_cov_max, circ_class_dist, wstress_season, &
       lm_lastyearmax, age, leaf_frac, atm_to_bm, &
       everywhere, litter, som, lignin_struc, &
       lignin_wood, bm_to_litter, turnover_daily,PFTpresent, when_growthinit,&
       forest_managed, KF, plant_status, &
       npp_longterm, gpp_daily, leaf_age, lab_fac, &
       gdd_from_growthinit, gdd_midwinter, time_hum_min, &
       hum_min_dormance, gdd_m5_dormance, ncd_dormance, &
       moiavail_month, moiavail_week, ngd_minus5, &
       resp_maint, resp_growth, npp_daily, &
       use_reserve, rue_longterm, mai, pai, &
       mai_count, previous_wood_volume, moiavail_growingseason, &
       MatrixA, MatrixV, VectorB, VectorU, age_stand, last_cut, &
       k_latosa_adapt, fm_change_map, lpft_replant, cn_leaf_min_season,&
       nstress_season, soil_n_min, p_O2, bact, store_sum_delta_ba,&
       CN_som_litter_longterm,nbp_pool_start )

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                                :: npts                !! Domain size - number of pixels (unitless)
    REAL(r_std), DIMENSION(:), INTENT(in)              :: circ_class_dist     !! The probability distribution of trees in a circ
                                                                              !! class in case of a redistribution (unitless).
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)         :: fm_change_map       !! A map which gives the desired FM strategy when
                                                                              !! the PFT will be replanted after a clearcut.
                                                                              !! (1-nvm,unitless)
  LOGICAL, DIMENSION(npts,nvm), INTENT(in)             :: lpft_replant        !! Set to true if a PFT has been clearcut
                                                                               !! and needs to be replaced by another species


    !! 0.2 Output variables


    !! 0.3 Modified variables

    LOGICAL, DIMENSION(:,:), INTENT(inout)             :: PFTpresent          !! Tab indicating which PFTs are present in 
                                                                              !! each pixel
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: plant_status        !! Growth and phenological status of the plant
                                                                              !! istatus = Phases defined in constantes
    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)     :: forest_managed      !! forest management flag (is the forest 
                                                                              !! being managed?)
     REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: moiavail_month      !! "Monthly" moisture availability (0 to 1, 
                                                                              !! unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: moiavail_week       !! "Weekly" moisture availability 
                                                                              !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: ngd_minus5          !! Number of growing days (days), threshold 
                                                                              !! -5 deg C (for phenology)   
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: resp_maint          !! Maintenance respiration  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: resp_growth         !! Growth respiration  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: npp_daily           !! Net primary productivity 
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: use_reserve         !! Mass taken from carbohydrate reserve 
                                                                              !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: rue_longterm        !! Longterm radiation use efficiency 
                                                                              !! (??units??) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: when_growthinit     !! How many days ago was the beginning of 
                                                                              !! the growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: lab_fac             !! Activity of labile pool factor (??units??)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: KF                  !! Scaling factor to convert sapwood mass
                                                                              !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: k_latosa_adapt      !! Leaf to sapwood area adapted for long 
                                                                              !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: npp_longterm        !! "Long term" mean yearly primary productivity
                                                                              !! @tex $(m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: veget_cov_max           !! "maximal" coverage fraction of a PFT on the ground
                                                                              !! May sum to
                                                                              !! less than unity if the pixel has
                                                                              !! nobio area. (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: wstress_season      !! Water stress factor, based on hum_rel_daily
                                                                              !! (unitless, 0-1)   
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: lm_lastyearmax      !! last year's maximum leaf mass for each PFT 
                                                                              !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: everywhere          !! is the PFT everywhere in the grid box or 
                                                                              !! very localized (after its introduction) (?)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: age                 !! mean age (years)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: atm_to_bm           !! CO2 and N taken from the atmosphere to get C to create  
                                                                              !! the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gpp_daily           !! Daily gross primary productivity  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: time_hum_min        !! Time elapsed since strongest moisture 
                                                                              !! availability (days) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)    :: hum_min_dormance    !! minimum moisture during dormance 
                                                                              !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_midwinter       !! Growing degree days (K), since midwinter 
                                                                              !! (for phenology) - this is written to the
                                                                              !!  history files 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_from_growthinit !! growing degree days, since growthinit 
                                                                              !! for crops
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_m5_dormance     !! Growing degree days (K), threshold -5 deg 
                                                                              !! C (for phenology)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: ncd_dormance        !! Number of chilling days (days), since 
                                                                              !! leaves were lost (for phenology) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: circ_class_n        !! Number of individuals in each circ class
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: lignin_struc        !! ratio Lignine/Carbon in structural litter,
                                                                              !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: lignin_wood         !! ratio Lignine/Carbon in woody litter,
                                                                              !! above and below ground
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: som                 !! carbon pool: active, slow, or passive 
                                                                              !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: leaf_frac           !! fraction of leaves in leaf age class (unitless;0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: leaf_age            !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: bm_to_litter        !! Transfer of biomass to litter 
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: turnover_daily        !! Transfer of litter to som 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)   :: circ_class_biomass  !! Biomass components of the model tree within a 
                                                                              !! circumference class
                                                                              !! class @tex $(g C ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)   :: litter              !! metabolic and structural litter, above and 
                                                                              !! below ground @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: mai                 !! The mean annual increment
                                                                              !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: pai                 !! The period annual increment
                                                                              !! @tex $(m**3 / m**2 / year)$ @endtex          
    INTEGER(i_std), DIMENSION(:,:),INTENT(inout)       :: mai_count           !! The number of times we've
                                                                              !! calculated the volume increment
                                                                              !! for a stand 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: previous_wood_volume!! The volume of the tree trunks
                                                                              !! in a stand for the previous year.
                                                                              !! @tex $(m**3 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: moiavail_growingseason  !! Mean growingseason moisture 
                                                                              !! availability (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: MatrixA             !! Matrix containing the fluxes between the
                                                                              !! carbon pools per sechiba time step 
                                                                              !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: VectorB             !! Vector containing the litter increase per
                                                                              !! sechiba time step
                                                                              !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: MatrixV             !!
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: VectorU             !!
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)      :: age_stand           !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)      :: last_cut            !! Years since last thinning (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: cn_leaf_min_season  !! Seasonal min CN ratio of leaves 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: nstress_season      !! N-related seasonal stress (used for allocation) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: soil_n_min          !! mineral nitrogen in the soil (gN/m**2)  
                                                                              !! (first index=npts, second index=nvm, third index=nnspec) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: p_O2                !! partial pressure of oxygen in the soil (hPa)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: bact                !! denitrifier biomass (gC/m**2)
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)        :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), DIMENSION(:,:),INTENT(inout)          :: nbp_pool_start
 
    !! 0.4 Local variables

    ! +++CHECK+++
    ! Externalize
     REAL(r_std), DIMENSION(nagec)                     :: age_class_bound     !! Boundaries of the age classes (m)
    ! +++++++++++
     
    INTEGER(i_std)                                     :: ipts,ivm,igroup     !! Indeces(unitless)
    INTEGER(i_std)                                     :: iele,ipar,ipft      !! Indeces(unitless)
    INTEGER(i_std)                                     :: iagec,imbc,icir     !! Indeces(unitless)
    INTEGER(i_std)                                     :: ilit,ilev,icarb     !! Indeces(unitless)
    INTEGER(i_std)                                     :: ivma,ispec          !! Indeces(unitless)
    INTEGER(i_std)                                     :: old_pft             !! pft number before merge
    INTEGER(i_std)                                     :: new_pft             !! pft number to merge with
    REAL(r_std)                                        :: share_expanded      !! Share of the veget_cov_max of the existing vegetation
                                                                              !! within a PFT over the total veget_cov_max following 
                                                                              !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(ncirc)                      :: circ_class_ba       !! basal area of the model tree in each
                                                                              !! circ class @tex $(m^{2} m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,ncirc)             :: store_sum_delta_ba  !! Sum of basal area increment (m^2)
    REAL(r_std), DIMENSION(npts,nvm)                   :: qm_dia              !! Quadratic mean diameter of the stand (m)
    REAL(r_std), DIMENSION(ncirc)                      :: est_circ_class_n    !! Temporary variable for circ_class_n of the 
                                                                              !! established vegetation 
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                      :: tmp_circ_class_n    !! Temporary variable for circ_class_n of the 
                                                                              !! established vegetation 
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)     :: est_circ_class_biomass !! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)     :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                      :: est_circ_class_dia  !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                      :: tmp_circ_class_dia  !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) :: check_intern        !! Contains the components of the internal
                                                                              !! mass balance chech for this routine
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: closure_intern      !! Check closure of internal mass balance
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_start          !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_end            !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nelements)                  :: temp_start          !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nelements)                  :: temp_end            !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nlitt,nlevs)                :: litter_weight_expanded !! The fraction of litter on the expanded
                                                                              !! PFT. @tex $-$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                   :: veget_cov_max_begin     !! Temporairy variable to check area conservation

!_ ================================================================================================================================
  
    IF (printlev.GE.3) WRITE(numout,*) 'Entering age class distribution'

 !! 1. Initialize
    
    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('age_class_distr')

    !! 1.1 Initialize variables
    !+++CHECK++++
    !Externalize
    IF(nagec == 4)THEN
       age_class_bound(1) = 0.07
       age_class_bound(2) = 0.20
       age_class_bound(3) = 0.40
       age_class_bound(4) = 5.
    ELSEIF(nagec == 1)THEN
       age_class_bound(1) = 5.0
    ELSE
       WRITE(numout,*) 'You need to externalize age_class_bound, or come up with a case '
       WRITE(numout,*) 'for this number of age classes.'
       WRITE(numout,*) 'nagec = ',nagec
       CALL ipslerr_p (3,'age_class_distr', 'Redefine age class bounds','','')
    ENDIF
    !++++++++++++

    !! 1.2 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 3. Initial biomass and harvest pool all other
    !  relevant pools were just set to zero.
    IF (err_act.GT.1) THEN

       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO iele = 1,nelements

          ! atm_to_bm
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               atm_to_bm(:,:,iele)
          
          !  Initial biomass + bm_to_litter
          DO ipar = 1,nparts

             DO icir = 1,ncirc

                ! Initial biomass pool
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:)

             ENDDO

             ! Add bm_to_litter to the initial biomass pool
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  ((bm_to_litter(:,:,ipar,iele)+ &
                  turnover_daily(:,:,ipar,iele)) * veget_cov_max(:,:))

          ENDDO

          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO
       
          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO
          
          ! Denitrifying bacteria (only C)
          pool_start(:,:,icarbon) = pool_start(:,:,icarbon) + &
                  bact(:,:) * veget_cov_max(:,:)

          ! Nitrogen only
          DO ispec = 1,nnspec
             pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
                  soil_n_min(:,:,ispec) * veget_cov_max(:,:)
          ENDDO

          

       ENDDO

       !! 1.3 Initialize check for area conservation
       veget_cov_max_begin(:,:) = veget_cov_max(:,:)
       
    ENDIF ! err_act.GT.1


 !! 2. Redistribute the biomass across the age classes

    !  Following growth, the trees from an age class may have become too big to 
    !  belong to this age class. The biomass, litter, soilcarbon and soil
    !  water then need to be moved from one age class to the next age class.
    DO ipts = 1,npts

       ! The agec_group lets us know which PFTs belong in which age class groups.
       !  I assume that the structure is linear, that is, if PFTs 16, 17, and 18 all
       ! below to group 10, PFT 16 is the youngest, PFT 17 is the second-youngest, and
       ! PFT 18 is the oldest (largest diameter).  I also assume that all PFTs in 
       ! an age class are grouped together in the input file.  In the example above,
       ! this means that PFT 19 cannot be in group 10 unless PFT 18 is also in group 10.       

       ! This loops over all the PFTs that we have while ignoring age classes
       DO ivma=1,nvmap

          ! start_index is an array with nvmap values wich gives the pft number
          ! of the youngest age class for the nvmap groups. Say we have 28 nvmap
          ! but run with a total of 64 pfts because some of the 28 groups have
          ! age class. then start_index will have 28 dimensions and the value of
          ! start_index will vary between 1 and 64.
          ivm=start_index(ivma)

          ! If we only have a single age class for this
          ! PFT, we can skip it. nagec_pft contains for each group either 1 
          ! (no age classes) or the number of age classes.
          IF(nagec_pft(ivma) .EQ. 1)CYCLE

          ! Only forest PFTs have age classes
          IF (is_tree(ivm)) THEN

             !! 2.1 Check if the trees still belong to this age class
             !  Note that the term age class is used but that the classes used in the
             !  code are not defined on an age criterion. Instead a diameter criterion
             !  is used. The reason to do so is that growth depends on the rule of
             !  Deleuze and Dhote that in turn is based on basal area. More stable
             !  model behaviour is therefore expected when the diameter range within an
             !  age class is controlled. Also different PFTs grow at different speeds.
             !  If age would have been used to define the classes, the diameter range
             !  within the first age class would have been much larger in the 
             !  tropics compared to the boreal zone. When Deleuze and Dhote is then
             !  applied the model could start to behave differently in the tropics and
             !  boreal zone. These differences in behaviour are thought to be better 
             !  controlled by implementing a diameter criterion.
             !  As a reminder age classes (::nagec) control the diameter/height
             !  variation across stands, ::ncirc controls the diameter/height variation
             !  within a stand
             DO iagec = nagec_pft(ivma),1,-1

                ipft = ivm+iagec-1

                ! If there are no trees, there is no need to merge
                IF (veget_cov_max(ipts,ipft).LT.min_stomate) CYCLE

                !! 2.1.1 Basal area at the tree level (m2 tree-1)
                circ_class_ba(:) = &
                     wood_to_ba(circ_class_biomass(ipts,ipft,:,:,icarbon),ipft)

                ! Convert basal area in quadratic mean stand diameter (m)
                IF (SUM(circ_class_n(ipts,ipft,:)) .NE. zero) THEN

                   qm_dia(ipts,ipft) = SQRT( 4/pi * SUM(circ_class_ba(:) * &
                        circ_class_n(ipts,ipft,:)) / &
                        SUM(circ_class_n(ipts,ipft,:)) )

                ELSE

                   qm_dia(ipts,ipft) = zero

                ENDIF

                !! 2.2.2 Check whether quadratic mean diameter exceeds boundaries 
                !  Check whether quadratic mean diameter exceeds boundaries of
                !  the age class.

                !Debug
                IF(printlev_loc>=4)THEN
                   WRITE(numout,*) 'Checking to merge for: '
                   WRITE(numout,*) 'ipft,ivm,iagec,ipts: ',ipft,ivm,iagec,ipts
                   WRITE(numout,*) 'nagec_pft,qm_dia,age_class_bound: ',nagec_pft(ivma),&
                        qm_dia(ipts,ipft),age_class_bound(iagec)
                ENDIF
                !-
                
                ! If we flagged a species to be replanted the biomass
                ! pools of both the new and the old pft can be empty.
                ! Redistributing empty pools doesn't make too much sense.
                ! Replanting is taken care of in species_change.
                IF(.NOT.lpft_replant(ipts,ipft))THEN

                   ! Debug
                   IF(printlev_loc>=4) WRITE(numout,*) 'Ready to redistribute'
                   !-

                   IF ( (iagec .EQ. nagec_pft(ivma)) .AND. &
                        qm_dia(ipts,ipft) .GT. age_class_bound(iagec) ) THEN
                      
                      ! If these conditions are satisfied our tree diameter is
                      ! very unrealist
                      WRITE(numout,*) 'WARNING: tree diameter exceeds (m): ', &
                           age_class_bound(iagec)
                      CALL ipslerr_p (3,'ERROR: age_class_distr',&
                           'tree diameter exceeds the threshold','','')

                      old_pft = ipft
                      new_pft = ipft
  
                   ELSEIF ( (iagec .NE. nagec_pft(ivma)) .AND. &
                        qm_dia(ipts,ipft) .GT. age_class_bound(iagec) ) THEN
                   
                      ! If this condition is satisfied, the trees are too
                      ! big for the current age class. They will be moved
                      ! one age class up
                      old_pft = ipft
                      new_pft = ipft+1

                   ELSEIF ( (iagec .NE. start_index(ivma)) .AND. &
                        qm_dia(ipts,ipft) .LT. age_class_bound(1) ) THEN

                      ! If this condition is satisfied, the trees are too
                      ! small for the current age class (after a harvest)
                      ! and they will be moved to the first age class. We 
                      ! don't want this to happen if we are in the first 
                      ! age class.
                      old_pft = ipft
                      new_pft = start_index(ivma)
                  
                      ! Debug
                      IF(printlev_loc>=4)THEN
                         WRITE(numout,*) 'age classes going down'
                         WRITE(numout,*) 'iagec, ipft, start_index, ', &
                              iagec, ipft, start_index(ivma)
                         WRITE(numout,*) 'qm_dia, ', qm_dia(ipts,ipft)
                         WRITE(numout,*) 'age_class_bound, ', age_class_bound(1)
                         WRITE(numout,*) 'old_pft, new_pft, ',old_pft, new_pft
                      ENDIF
                      !-

                   ELSE

                      ! There is no need to change the age class. Set 
                      ! old and new pft to the same value. That way
                      ! the next IF-loop is skipped.
                      old_pft = zero
                      new_pft = zero
                      
                   ENDIF ! age_class_bounds

                ELSE

                   ! lpft_replant is TRUE. Don't change the age class. 
                   ! Set old and new pft to the same value. That way
                   ! the next IF-loop is skipped.
                   old_pft = zero
                   new_pft = zero

                ENDIF !lpft_replant

                ! Debug
                IF(printlev_loc>=4 .AND. old_pft .NE. new_pft)THEN
                   WRITE(numout,*) 'Merging biomass'
                   WRITE(numout,*) 'ipts,ipft,iagec: ',ipts,ipft,iagec
                   WRITE(numout,*) 'age_class_bound(:): ',age_class_bound(:)
                   WRITE(numout,*) 'qm_dia: ',qm_dia(ipts,ipft)
                   WRITE(numout,*) 'old_pft, new_pft, ',old_pft, new_pft
                ENDIF
                !-

                IF (old_pft .NE. new_pft) THEN

                   !! 2.2.3 Merge biomass
                   !  Biomass of two age classes needs to be merged. The established
                   !  vegetation is stored in new_pft, the new vegetation is stored in
                   !  old_pft
                   share_expanded = veget_cov_max(ipts,new_pft) / &
                        ( veget_cov_max(ipts,new_pft) + veget_cov_max(ipts,old_pft) )

                   ! We also need a scaling factor which includes the litter
                   DO ilev=1,nlevs

                      DO ilit=1,nlitt

                         IF(litter(ipts,ilit,old_pft,ilev,icarbon).GE.min_stomate) THEN

                            ! There is C and N in the litter. Calculate a weighted
                            ! mean
                            litter_weight_expanded(ilit,ilev) = &
                                 litter(ipts,ilit,new_pft,ilev,icarbon) * &
                                 veget_cov_max(ipts,new_pft)/ &
                                 (litter(ipts,ilit,new_pft,ilev,icarbon) * &
                                 veget_cov_max(ipts,new_pft) + &
                                 litter(ipts,ilit,old_pft,ilev,icarbon) * &
                                 veget_cov_max(ipts,old_pft))

                         ELSEIF (litter(ipts,ilit,old_pft,ilev,icarbon).LT.min_stomate) THEN

                            ! The litter pools are empty
                            litter_weight_expanded(ilit,ilev) = zero

                         ELSE

                            ! There is either no carbon or no nitrogen in the
                            ! litter. This is strange as one should always come
                            ! with the other.
                            WRITE(numout,*) 'ERROR: age_class_distr'
                            WRITE(numout,*) 'There is litter but the C/N ratio'
                            WRITE(numout,*) 'does not make any sense'
                            WRITE(numout,*) 'C in the litter', ipts, ilit, old_pft, ilev, &
                                 litter(ipts,ilit,old_pft,ilev,icarbon)
                            WRITE(numout,*) 'N in the litter', ipts, ilit, old_pft, ilev, &
                                 litter(ipts,ilit,old_pft,ilev,initrogen)
                            CALL ipslerr_p(3,'ERROR: age_class_distr',&
                                 'There is litter but the C/N ratio',&
                                 'does not make any sense','')

                         ENDIF

                      END DO !nlitt

                   ENDDO ! nlevs
     
                   ! Assign value to moiavail_growingseason, this value needs to be 
                   ! copied because it is only calculated when there is leaf mass present 
                   moiavail_growingseason(ipts,new_pft) = &
                        moiavail_growingseason(ipts,new_pft) * &
                        share_expanded + moiavail_growingseason(ipts,old_pft) * &
                        (un - share_expanded)

                   ! I'm not sure if wstress_season needs to be 
                   ! here since it is recalculated from
                   ! moiavail_growingseason every day.
                   wstress_season(ipts,new_pft) = wstress_season(ipts,new_pft) * &
                          share_expanded + wstress_season(ipts,old_pft) * (un - share_expanded)

                   ! Copy the number of individuals and the biomass of the established 
                   ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                   ! and ::circ_class_biomass will be overwritten with the characteristics
                   ! of the merged vegetation
                   est_circ_class_n(:) = circ_class_n(ipts,new_pft,:)
                   est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,new_pft,:,:,:)
                   tmp_circ_class_n(:) = circ_class_n(ipts,old_pft,:)
                   tmp_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,old_pft,:,:,:)
 
                   ! Store circumference (m) in a temporary variable that can be 
                   ! sorted from small to large - note that the dimension of these
                   ! variables are 2*ncirc
                   est_circ_class_dia(:) = wood_to_dia(est_circ_class_biomass(:,:,icarbon),&
                        new_pft)
                   tmp_circ_class_dia(:) = wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),&
                        old_pft)


                   ! Debug
                   ! Initiliaze mass balance of the merging procedure
                   ! MB was calculated outside the subroutine merge_biomass_pfts
                   ! because veget_cov_max needs to be updated. Veget_max cannot be 
                   ! updated in the subroutine because the way it is called from 
                   ! the subroutine land_cover_change (where veget_cov_max can only be
                   ! updated at the end of the whole subroutine. Therefore, this
                   ! check was included here.
                   IF (printlev_loc>=4) THEN
                      temp_start(:) = zero
                      DO ipar = 1,nparts
                         DO iele = 1,nelements
                            DO icir = 1,ncirc
                               temp_start(iele) = temp_start(iele) + &
                                    SUM(circ_class_biomass(ipts,:,icir,ipar,iele) * &
                                    circ_class_n(ipts,:,icir) * veget_cov_max(ipts,:))
                            ENDDO
                         ENDDO
                      ENDDO
                      ! Some output - unrelated to the mass balance
                      WRITE(numout,*) 'age distribution share_expanded, ',ipts, ivm, &
                           share_expanded
                      WRITE(numout,*) 'rest, ', SUM(circ_class_n(ipts,ivm,:)), &
                           SUM(est_circ_class_n), SUM(tmp_circ_class_n), &
                           SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2),1), &
                           SUM(est_circ_class_biomass), &
                           SUM(tmp_circ_class_biomass), &
                           ipts, new_pft, SUM(est_circ_class_dia), SUM(tmp_circ_class_dia), &
                           lpft_replant(ipts,ivm)
                      WRITE(numout,*) 'old_pft, new_pft, ',old_pft, new_pft
                   ENDIF
                   !-
                   
                   ! Merge the biomass of the two age classes
                   CALL merge_biomass_pfts(npts, share_expanded, circ_class_n, &
                        est_circ_class_n, tmp_circ_class_n, circ_class_biomass, &
                        est_circ_class_biomass, &
                        tmp_circ_class_biomass, circ_class_dist, ipts, &
                        new_pft, est_circ_class_dia, tmp_circ_class_dia)

                   !! 2.2.4 Empty the age class that was merged and update veget_cov_max
                   circ_class_n(ipts,old_pft,:) = zero
                   circ_class_biomass(ipts,old_pft,:,:,:) = zero
                   veget_cov_max(ipts,new_pft) = veget_cov_max(ipts,new_pft) + &
                        veget_cov_max(ipts,old_pft)
                   veget_cov_max(ipts,old_pft) = zero

                   ! Debug
                   IF (printlev_loc>=4) THEN
                      ! Mass balance of the merging procedure
                      temp_end(:) = zero
                      DO ipar = 1,nparts
                         DO iele = 1,nelements
                            DO icir = 1,ncirc
                               temp_end(iele) = temp_end(iele) + &
                                    SUM(circ_class_biomass(ipts,:,icir,ipar,iele) * &
                                    circ_class_n(ipts,:,icir) * veget_cov_max(ipts,:))
                            ENDDO
                         ENDDO
                      ENDDO
 
                      DO iele = 1,nelements
                         IF (temp_end(iele) - temp_start(iele) .LT. min_stomate .AND. &
                              temp_end(iele) - temp_start(iele) .GT. -min_stomate) THEN
                            WRITE(numout,*) 'Mass balance closure merge_biomass_pfts', iele
                         ELSE
                            WRITE(numout,*) 'Error: mass balance is not closed in merge_biomass_pfts'
                            WRITE(numout,*) '   Difference is, ',iele, temp_end(iele) - &
                                 temp_start(iele), temp_end(iele), &
                                 temp_start(iele) 
                            WRITE(numout,*) 'carbon ', &
                                 SUM(SUM(circ_class_biomass(ipts,:,:,:,icarbon),3),2)
                            WRITE(numout,*) 'nitrogen ', &
                                 SUM(SUM(circ_class_biomass(ipts,:,:,:,initrogen),3),2)
                            WRITE(numout,*) 'veget_cov_max, ',veget_cov_max(ipts,:)
                            IF (err_act.GT.1) THEN
                               CALL ipslerr_p(3,'ERROR:mass balance not closed',&
                                    'after merge biomass in age_class_distr','','')
                            ENDIF
                         ENDIF
                      ENDDO !nelements
                   ENDIF
                   !-

                   !! 2.3 Calculate the PFT characteristics of the merged PFT
                   !  Take the weighted mean of the existing vegetation and the new 
                   !  vegetation in this PFT. Note that atm_to_bm is in gC. m-2 dt-1 
                   !  so we should also take the weighted mean (rather than the sum if
                   !  this where absolute values).
                   lm_lastyearmax(ipts,new_pft) = share_expanded * &
                        lm_lastyearmax(ipts,new_pft) + &
                        (un - share_expanded) * lm_lastyearmax(ipts,old_pft)
                   lm_lastyearmax(ipts,old_pft) = zero
                   age(ipts,new_pft) = share_expanded * age(ipts,new_pft) + &
                        (un - share_expanded) * age(ipts,old_pft)
                   age(ipts,old_pft) = zero
                   leaf_frac(ipts,new_pft,:) = share_expanded * leaf_frac(ipts,new_pft,:) + &
                        (un - share_expanded) * leaf_frac(ipts,old_pft,:)
                   leaf_frac(ipts,old_pft,:) = zero
                   leaf_age(ipts,new_pft,:) = share_expanded * leaf_age(ipts,new_pft,:) + &
                        (un - share_expanded) * leaf_age(ipts,old_pft,:)
                   leaf_age(ipts,old_pft,:) = zero
                   lab_fac(ipts,new_pft) = share_expanded * lab_fac(ipts,new_pft) + &
                        (un - share_expanded) * lab_fac(ipts,old_pft)
                   lab_fac(ipts,old_pft) = zero
                   atm_to_bm(ipts,new_pft,:) = share_expanded * atm_to_bm(ipts,new_pft,:) + &
                        (un - share_expanded) * atm_to_bm(ipts,old_pft,:)
                   atm_to_bm(ipts,old_pft,:) = zero
                   everywhere(ipts,new_pft) = MAX(everywhere(ipts,old_pft), &
                        everywhere(ipts,new_pft))
                   everywhere(ipts,old_pft) = zero
                   nstress_season(ipts,new_pft) = share_expanded * &
                        nstress_season(ipts,new_pft) + &
                        (un - share_expanded) * nstress_season(ipts,old_pft)
                   nstress_season(ipts,old_pft) = zero
                   cn_leaf_min_season(ipts,new_pft) = share_expanded * &
                        cn_leaf_min_season(ipts,new_pft) + &
                        (un - share_expanded) * cn_leaf_min_season(ipts,old_pft)
                   cn_leaf_min_season(ipts,old_pft) = zero

                   ! The new soil&litter pools are the weighted mean of the newly 
                   ! established vegetation for that PFT and the soil&litter pools
                   ! of the vegetation that already exists in that PFT.
                   ! Since it is not only the amount of vegetation present (veget_cov_max) but also
                   ! the amount of structural litter (litter) that is important, we have to 
                   ! weight by both items here.
                   DO ilev=1,nlevs
                      lignin_struc(ipts,new_pft,ilev) = &
                           litter_weight_expanded(istructural,ilev) * &
                           lignin_struc(ipts,new_pft,ilev) + &
                           (un - litter_weight_expanded(istructural,ilev)) * &
                           lignin_struc(ipts,old_pft,ilev) 
                      lignin_struc(ipts,old_pft,ilev) = zero
                      lignin_wood(ipts,new_pft,ilev) = &
                           litter_weight_expanded(iwoody,ilev) * &
                           lignin_wood(ipts,new_pft,ilev) + &
                           (un - litter_weight_expanded(iwoody,ilev) ) * &
                           lignin_wood(ipts,old_pft,ilev) 
                      lignin_wood(ipts,old_pft,ilev) = zero 
                   ENDDO
                   
                   litter(ipts,:,new_pft,:,:) = share_expanded * litter(ipts,:,new_pft,:,:) + &
                        (un - share_expanded) * litter(ipts,:,old_pft,:,:)
                   litter(ipts,:,old_pft,:,:) = zero
                   som(ipts,:,new_pft,:) =  share_expanded * som(ipts,:,new_pft,:) + &
                        (un - share_expanded) * som(ipts,:,old_pft,:)
                   som(ipts,:,old_pft,:) = zero 
                   bm_to_litter(ipts,new_pft,:,:) = share_expanded * &
                        bm_to_litter(ipts,new_pft,:,:) + & 
                        (un - share_expanded) * bm_to_litter(ipts,old_pft,:,:)
                   bm_to_litter(ipts,old_pft,:,:) = zero
                   turnover_daily(ipts,new_pft,:,:) = share_expanded * &
                        turnover_daily(ipts,new_pft,:,:) + &
                        (un - share_expanded) * turnover_daily(ipts,old_pft,:,:)
                   turnover_daily(ipts,old_pft,:,:) = zero
                   soil_n_min(ipts,new_pft,:) = share_expanded * &
                        soil_n_min(ipts,new_pft,:) + & 
                        (un - share_expanded) * soil_n_min(ipts,old_pft,:)
                   soil_n_min(ipts,old_pft,:) = zero
                   p_O2(ipts,new_pft) = share_expanded * p_O2(ipts,new_pft) + & 
                        (un - share_expanded) * p_O2(ipts,old_pft)
                   p_O2(ipts,old_pft) = zero
                   bact(ipts,new_pft) = share_expanded * bact(ipts,new_pft) + & 
                        (un - share_expanded) * bact(ipts,old_pft)
                   bact(ipts,old_pft) = zero
                   CN_som_litter_longterm(ipts,new_pft,:) = &
                        share_expanded * CN_som_litter_longterm(ipts,new_pft,:) + &
                        (un - share_expanded) * CN_som_litter_longterm(ipts,old_pft,:)
                   CN_som_litter_longterm(ipts,old_pft,:) = zero
                   nbp_pool_start(ipts,new_pft) = &
                        share_expanded * nbp_pool_start(ipts,new_pft)+ &
                        (un - share_expanded) *nbp_pool_start(ipts,old_pft)
                   nbp_pool_start(ipts,old_pft) = zero



                   ! Copy variables that depend on ::when_growthinit
                   when_growthinit(ipts,new_pft) = share_expanded * &
                        when_growthinit(ipts,new_pft) + &
                        (un - share_expanded) * when_growthinit(ipts,old_pft)
                   when_growthinit(ipts,old_pft) = zero
                   gdd_from_growthinit(ipts,new_pft) = share_expanded * &
                        gdd_from_growthinit(ipts,new_pft) + &
                        (un - share_expanded) * gdd_from_growthinit(ipts,old_pft)
                   gdd_from_growthinit(ipts,old_pft) = zero
                   gdd_midwinter(ipts,new_pft) = share_expanded * &
                        gdd_midwinter(ipts,new_pft) + &
                        (un - share_expanded) * gdd_midwinter(ipts,old_pft)
                   gdd_midwinter(ipts,old_pft) = zero
                   time_hum_min(ipts,new_pft) = share_expanded * time_hum_min(ipts,new_pft) + &
                        (un - share_expanded) * time_hum_min(ipts,old_pft)
                   time_hum_min(ipts,old_pft) = zero
                   hum_min_dormance(ipts,new_pft) = share_expanded * hum_min_dormance(ipts,new_pft) + &
                        (un - share_expanded) * hum_min_dormance(ipts,old_pft)
                   hum_min_dormance(ipts,old_pft) = zero
                   gdd_m5_dormance(ipts,new_pft) = share_expanded * &
                        gdd_m5_dormance(ipts,new_pft) + &
                        (un - share_expanded) * gdd_m5_dormance(ipts,old_pft)
                   gdd_m5_dormance(ipts,old_pft) = zero
                   ncd_dormance(ipts,new_pft) = share_expanded * &
                        ncd_dormance(ipts,new_pft) + &
                        (un - share_expanded) * ncd_dormance(ipts,old_pft)
                   ncd_dormance(ipts,old_pft) = zero
                   moiavail_month(ipts,new_pft) = share_expanded * & 
                        moiavail_month(ipts,new_pft) + &
                        (un - share_expanded) * moiavail_month(ipts,old_pft)
                   moiavail_month(ipts,old_pft) = zero
                   moiavail_week(ipts,new_pft) = share_expanded * &
                        moiavail_week(ipts,new_pft) + &
                        (un - share_expanded) * moiavail_week(ipts,old_pft)
                   moiavail_week(ipts,old_pft) = zero
                   ngd_minus5(ipts,new_pft) = share_expanded * ngd_minus5(ipts,new_pft) + &
                        (un - share_expanded) * ngd_minus5(ipts,old_pft)
                   ngd_minus5(ipts,old_pft) = zero

                   ! When merging age classes and accounting for management 
                   ! changes, the merged pft is managed the same way as the new
                   ! pft. If not, prescribed changes in forest management (as 
                   ! part of the species change code) would not be consistently
                   ! implemented. However, when simply merging age classes without
                   ! species/management changes a conservative approach is
                   ! taken and the forest management strategy of the oldest age
                   ! class in the merge is used. 
                   IF(ok_change_species)THEN

                      ! If the old and new pft have a different management 
                      ! strategy we will have to decide which strategy we 
                      ! follow
                      IF (forest_managed(ipts,new_pft) .NE. &
                           forest_managed(ipts,old_pft)) THEN
                         
                         IF ((forest_managed(ipts,new_pft).EQ.ifm_none) .OR. &
                              (forest_managed(ipts,old_pft)).EQ.ifm_none) THEN

                            ! We assume that unmanaged sites are well 
                            ! protected. Actually they will even increase
                            ! in surface area even if the desired fm
                            ! strategy is not ifm_none.
                            forest_managed(ipts,new_pft) = ifm_none 
                            forest_managed(ipts,old_pft) = ifm_none

                         ELSE
                            
                            ! If the new old fm type are not equal we will 
                            ! use the new prescribed forest management as prepared
                            ! in fm_change_map. This should ensure that we can
                            ! get the old management back.
                            forest_managed(ipts,new_pft) = fm_change_map(ipts,new_pft) 
                            forest_managed(ipts,old_pft) = forest_managed(ipts,old_pft)

                            ! Debug
                            IF(printlev_loc>=4)THEN
                               WRITE(numout,*) 'species change - age class distr'
                               WRITE(numout,*) 'fm - old PFT, ',old_pft, &
                                    forest_managed(ipts,old_pft)
                               WRITE(numout,*) 'fm - new PFT, ',new_pft, &
                                    fm_change_map(ipts,new_pft)
                            ENDIF
                            !-
                            
                         ENDIF
                         
                      ELSE
                         
                         ! fm type for the new and old PFT are the same so nothing
                         ! should be done. The code was written to make it explicit
                         forest_managed(ipts,new_pft) = forest_managed(ipts,new_pft)
                         forest_managed(ipts,old_pft) = forest_managed(ipts,old_pft)

                      ENDIF

                    
                   ELSE ! no species change 

                      ! Preserve the prescribed forest management.
                      ! Do nothing but the code was written to make it explicit
                      forest_managed(ipts,new_pft) = forest_managed(ipts,new_pft)
                      forest_managed(ipts,old_pft) = forest_managed(ipts,old_pft)

                   END IF ! species change (through replant)?
                   
                   ! Copy remaining properties 
                   PFTpresent(ipts,new_pft) = PFTpresent(ipts,old_pft)
                   PFTpresent(ipts,old_pft) = .FALSE.
                   KF(ipts,new_pft) = share_expanded * KF(ipts,new_pft) + &
                        (un - share_expanded) * KF(ipts,old_pft)
                   KF(ipts,old_pft) = zero
                   k_latosa_adapt(ipts,new_pft) = share_expanded * &
                        k_latosa_adapt(ipts,new_pft) + &
                        (un - share_expanded) * k_latosa_adapt(ipts,old_pft)
                   k_latosa_adapt(ipts,old_pft) = zero
                   plant_status(ipts,new_pft) = plant_status(ipts,old_pft)
                   plant_status(ipts,old_pft) = iprescribe
                   npp_longterm(ipts,new_pft) = share_expanded * npp_longterm(ipts,new_pft) + &
                        (un - share_expanded) * npp_longterm(ipts,old_pft)
                   npp_longterm(ipts,old_pft) = zero
                   gpp_daily(ipts,new_pft) = share_expanded * gpp_daily(ipts,new_pft) + &
                        (un - share_expanded) * gpp_daily(ipts,old_pft)
                   gpp_daily(ipts,old_pft) = zero 
                   resp_maint(ipts,new_pft) = share_expanded * resp_maint(ipts,new_pft) + &
                        (un - share_expanded) * resp_maint(ipts,old_pft) 
                   resp_maint(ipts,old_pft) = zero
                   resp_growth(ipts,new_pft) = share_expanded * resp_growth(ipts,new_pft) + &
                        (un - share_expanded) * resp_growth(ipts,old_pft) 
                   resp_growth(ipts,old_pft) = zero
                   npp_daily(ipts,new_pft) = share_expanded * npp_daily(ipts,new_pft) + &
                        (un - share_expanded) * npp_daily(ipts,old_pft) 
                   npp_daily(ipts,old_pft) = zero
                   use_reserve(ipts,new_pft) = share_expanded * use_reserve(ipts,new_pft) + &
                        (un - share_expanded) * use_reserve(ipts,old_pft) 
                   use_reserve(ipts,old_pft) = zero
                   rue_longterm(ipts,new_pft)  = share_expanded * rue_longterm(ipts,new_pft) + &
                        (un - share_expanded) * rue_longterm(ipts,old_pft)
                   rue_longterm(ipts,old_pft) = zero
                   DO icir=1,ncirc
                        store_sum_delta_ba(ipts,new_pft,icir) = & 
                                share_expanded * store_sum_delta_ba(ipts,new_pft,icir) + &
                                (un - share_expanded) * store_sum_delta_ba(ipts,old_pft,icir)
                   ENDDO

                   ! Some things for forestry
                   mai(ipts,new_pft)  = share_expanded * mai(ipts,new_pft) + &
                        (un - share_expanded) * mai(ipts,old_pft)
                   mai(ipts,old_pft) = zero
                   pai(ipts,new_pft)  = share_expanded * pai(ipts,new_pft) + &
                        (un - share_expanded) * pai(ipts,old_pft)
                   pai(ipts,old_pft) = zero

                   ! We need an integer for mai_count.  The use of CEILING here is
                   ! arbitrary.  Maybe FLOOR would work better in some cases.
                   mai_count(ipts,new_pft)  = CEILING(share_expanded * &
                        mai_count(ipts,new_pft) + &
                        (un - share_expanded) * mai_count(ipts,old_pft))
                   mai_count(ipts,old_pft) = 0
                   previous_wood_volume(ipts,new_pft)  = share_expanded * &
                        previous_wood_volume(ipts,new_pft) + &
                        (un - share_expanded) * previous_wood_volume(ipts,old_pft)
                   previous_wood_volume(ipts,old_pft) = zero
                   
                   ! These next two variables are integers, so we use CEILING to make sure
                   ! they stay integers.  FLOOR or INT would also be fine.  The use of
                   ! CEILING is arbitrary.
                   age_stand(ipts,new_pft)  = CEILING(share_expanded * &
                        age_stand(ipts,new_pft) + &
                        (un - share_expanded) * age_stand(ipts,old_pft))
                   age_stand(ipts,old_pft) = zero
                   last_cut(ipts,new_pft)  = CEILING(share_expanded * last_cut(ipts,new_pft) + &
                        (un - share_expanded) * last_cut(ipts,old_pft))
                   last_cut(ipts,old_pft) = zero

                   ! If we are doing an analytical spinup, we need to merge
                   ! the matrices.  During our spinup, only one age class in a given
                   ! PFT will be populated.  Therefore, we just need to move
                   ! the matrix from one PFT to the next.  If there is a chance that
                   ! there is already biomass in the new PFT and things need to be merged,
                   ! you should rethink this!
                   IF (spinup_analytic) THEN
                      IF(share_expanded .LT. min_stomate)THEN
                         MatrixV(ipts,new_pft,:,:)=MatrixV(ipts,old_pft,:,:)
                         MatrixA(ipts,new_pft,:,:)=MatrixA(ipts,old_pft,:,:)
                         VectorU(ipts,new_pft,:)=VectorU(ipts,old_pft,:)
                         VectorB(ipts,new_pft,:)=VectorB(ipts,old_pft,:)
                         MatrixV(ipts,old_pft,:,:)=zero
                         MatrixA(ipts,old_pft,:,:)=zero
                         VectorU(ipts,old_pft,:)=zero
                         VectorB(ipts,old_pft,:)=zero
                      ELSE
                         WRITE(numout,*) 'Merge problem with spinup.'
                         WRITE(numout,*) 'ipts,ipft: ',ipts,old_pft
                         WRITE(numout,*) 'share_expanded: ',share_expanded
                         CALL ipslerr (3,'age_class_distr', &
                              'Trying to merge spinup matrices into a PFT with existing',&
                              'biomass. Stopping because we are not sure what to do.','')
                      ENDIF
                   ENDIF

                ELSE

                   ! Debug
                   IF (printlev_loc>=4) WRITE(numout,*) 'old and new pft are the same'
                   !-

                ENDIF

             ENDDO ! loop over nagec_pft(ivma)

          ELSE ! Grasses and croplands

             ! Don't have to do anything

             ! Debug
             IF (printlev_loc>=4) WRITE(numout,*) 'Grass or cropland ',&
                  'no age classes', ivm
             !-

          ENDIF ! is_tree(ivm)

       ENDDO ! Looping over real PFTs

    ENDDO ! loop over #pixels - domain size


!! 3. Check numerical consistency of this routine

    IF (err_act.GT.1) THEN

       ! 4.2 Check surface area
       CALL check_surface_area("age class distr", npts, veget_cov_max_begin, &
            veget_cov_max,'ageclass')

       ! 4.3 Mass balance closure 
       pool_end = zero
       DO iele = 1,nelements

          ! atm_to_bm
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               atm_to_bm(:,:,iele)

          ! Biomass pool + bm_to_litter
          DO ipar = 1,nparts

             DO icir = 1,ncirc

                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:)

             ENDDO

             ! Add bm_to_litter to the biomass pool
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  ((bm_to_litter(:,:,ipar,iele)+ &
                  turnover_daily(:,:,ipar,iele)) * veget_cov_max(:,:))

          ENDDO
          
          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO

          ! Denitrifying bacteria (only C)
          pool_end(:,:,icarbon) = pool_end(:,:,icarbon) + &
                  bact(:,:) * veget_cov_max(:,:)

          ! Nitrogen only
          DO ispec = 1,nnspec
             pool_end(:,:,initrogen) = pool_end(:,:,initrogen) + &
                  soil_n_min(:,:,ispec) * veget_cov_max(:,:)
          ENDDO

       ENDDO
       
       ! 4.3.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
               pool_start(:,:,iele)) 
       ENDDO

       closure_intern(:,:,:) = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             ! Debug
             IF (printlev_loc>=4) &
                   WRITE(numout,*) &
                  'check_intern, ivm, imbc, iele, ', imbc, &
                  iele, check_intern(:,test_pft,imbc,iele)
             !-
             closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
                  check_intern(:,:,imbc,iele)
          ENDDO
       ENDDO

       ! 4.3.3 Check mass balance closure
       CALL check_mass_balance("age class distr", &
            closure_intern, npts, pool_end, pool_start, veget_cov_max, &
            'ageclass')
      
    ENDIF ! err_act.GT.1

    IF(printlev.GE.4) WRITE(numout,*) 'Leaving age class distribution'
    
  END SUBROUTINE age_class_distr


! ================================================================================================================================
!! SUBROUTINE   : merge_biomass_pfts
!!
!>\BRIEF        Redistribute biomass, litter, soilcarbon and water across
!!              the age classes
!!
!! DESCRIPTION  : Following growth, the trees from an age class may have too
!! big to still belong to this age class. The biomass, litter, soilcarbon and
!! soil water then need to be moved from one age class to the next age class.
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE merge_biomass_pfts(npts, share_expanded, circ_class_n, est_circ_class_n, &
       tmp_circ_class_n, circ_class_biomass, est_circ_class_biomass, &
       tmp_circ_class_biomass, circ_class_dist, ipts, &
       ivm, est_circ_class_dia, tmp_circ_class_dia)

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
    
    INTEGER, INTENT(in)                              :: npts                  !! Domain size - number of pixels (unitless)
    REAL(r_std), INTENT(in)                          :: share_expanded        !! Share of the veget_cov_max of the existing vegetation
                                                                              !! within a PFT over the total veget_cov_max following 
                                                                              !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: circ_class_dist       !! The probability distribution of trees in a circ
                                                                              !! class in case of a redistribution (unitless). 
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: est_circ_class_biomass!! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: tmp_circ_class_biomass!! Temporary variable for circ_class_biomass 
                                                                              !! of the new vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: est_circ_class_n      !! Temporary variable for circ_class_n
                                                                              !! of the established vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tmp_circ_class_n      !! Temporary variable for circ_class_n
                                                                              !! of the new vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: est_circ_class_dia    !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tmp_circ_class_dia    !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    INTEGER(i_std), INTENT(IN)                       :: ipts, ivm             !! Indices for point and PFT (unitless)

    !! 0.2 Output variables


    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass    !! circ_class_biomass of the merged vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n          !! circ_class_n of the merged vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex

    !! 0.4 Local variables
    INTEGER(i_std)                                   :: ipart                 !! Indices (unitless)
    INTEGER(i_std)                                   :: icir, jcir, iele      !! Indices (unitless)
    REAL(r_std)                                      :: trees_needed          !! Number of trees leftthat should be present in a 
                                                                              !! circumference class when merging the established
                                                                              !! and newly established vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex 
    REAL(r_std)                                      :: tempa, tempb          !! Variable to store a temporary value (unit varies)
    REAL(r_std), DIMENSION(nparts,nelements)         :: tempc                 !! Variable to store a temporary value of
                                                                              !! circ_class_biomass  
    REAL(r_std), DIMENSION(ncirc)                    :: exp_circ_class_n      !! Temporary variable for circ_class_n of the
                                                                              !! expanded PFT @tex $(ind m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(2*ncirc)                  :: trees_left            !! Number of trees left in a circumference class when
                                                                              !! merging the available and newly established 
                                                                              !! vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(2*ncirc)                  :: temp_circ_class       !! Temporary variable for circ_class of the expanded
                                                                              !! PFT for sorting the availble and newly established
                                                                              !! circumference classes (m)
    REAL(r_std), DIMENSION(2*ncirc)                  :: temp_circ_class_n     !! Temporary variable for circ_class_n of the expanded
                                                                              !! PFT for sorting the availble and newly established
                                                                              !! circumference classes @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(2*ncirc,nparts,nelements) :: temp_circ_class_biomass !! Temporary variable for sorting circ_class_biomass
                                                                              !! @tex $(g C ind^{-1})$ @endtex

!_ ================================================================================================================================

 IF (printlev.GE.3) WRITE(numout,*) 'Entering merging biomass of pfts'

 !! 1. Initialize

 !! 2. Merge biomass, litter, soilcarbon and soil water

    !  We have already ::ncirc classes in this PFT and we add another 
    !  ::ncirc through calling stomate_prescribe. What we want to 
    !  do is to merge these 2*ncirc into ncirc circumference classes.     
    !  We need to populate the new distribution of trees among
    !  the circumference classes, and move the heartwood and sap masses 
    !  to the new distributions. We also need to move the other pools.  
    !  This is more tricky because the allometric relations for the 
    !  new tree sizes will NOT give the same amount of biomass for
    !  the non-woody pools. Let us first try it just distributing
    !  everything equally, and then if that doesn't work we can
    !  redistribute the non-woody pools more cleverly, even if that 
    !  means we change the total amount of biomass we have in this
    !  stand.  I want to try it this way because we conserve biomass 
    !  this way, and I hope that the stress on the trees will be
    !  small enough that it doesn't cause problems. Redistribution
    !  is not expected to de-stabilize the allocation too much if the
    !  diameter range within an age-class is not too big (or in other
    !  words if we define a sufficiently large number of age classes).
 
    !! 2.1 Calculate new distribution
    !  The first step is to decide of the distribution of individuals 
    !  that we want in each class, for example, a uniform or exponential 
    !  distribution (::circ_class_dist). The total number of individuals
    !  that needs to be distributed is the weighted sum of the number of 
    !  individuals (#/m-2) that is already available the the 
    !  individuals that are newly established.  
    IF (share_expanded .LT. min_stomate) THEN

       ! The new PFT is still empty so simply copy the current tree 
       ! distribution into the new PFT
       exp_circ_class_n(:) = tmp_circ_class_n(:)

    ELSE

       ! The new PFT contains trees, merge the tree distributions. The
       ! new tree distribution follows a prescribed distribution
       exp_circ_class_n(:) = circ_class_dist(:) * &
            SUM( share_expanded * est_circ_class_n(:) + &
            (un-share_expanded) * tmp_circ_class_n(:) )

    ENDIF   


    !! 2.2 Copy both distributions into a single variable
    DO icir = 1,2*ncirc

       IF (icir .LE. ncirc) THEN

          ! Circumference classes, number of individuals and biomass
          ! of the available vegetation (m). Weight by the area share
          ! such that we don't need to do this further down. Don't
          ! weight the circumference and biomass as those variables are
          ! expressed per tree. This helps to simplify the equations.
          temp_circ_class(icir) = pi * est_circ_class_dia(icir)
          temp_circ_class_n(icir) = share_expanded * &
               est_circ_class_n(icir)
          temp_circ_class_biomass(icir,:,:) = &
               est_circ_class_biomass(icir,:,:)  

       ELSEIF ( (icir .GT. ncirc) .AND. (icir .LE. 2*ncirc) ) THEN

          ! Circumference classes, number of individuals and biomass
          ! of the newly established vegetation (m). Weight by the area 
          ! share such that we don't need to do this further down. Don't
          ! weight the circumference and biomass as those variables are
          ! expressed per tree. This helps to simplify the equations.
          temp_circ_class(icir) = pi * tmp_circ_class_dia(icir-ncirc)
          temp_circ_class_n(icir) = (un - share_expanded) * &
               tmp_circ_class_n(icir-ncirc)
          temp_circ_class_biomass(icir,:,:) = &
               tmp_circ_class_biomass(icir-ncirc,:,:)

       ELSE

          WRITE(numout,*) 'Error - problem with the dimensions of a do-loop '

       ENDIF

    ENDDO !loop over 2*ncirc


    !! 2.3 Sort the biomass and number of trees according to their 
    !  circumference 
    DO jcir = 2,2*ncirc

       tempa = temp_circ_class(jcir)
       tempb = temp_circ_class_n(jcir)
       tempc(:,:) = temp_circ_class_biomass(jcir,:,:)

       DO icir = jcir-1,1,-1

          IF (temp_circ_class(icir) .LE. tempa) EXIT
          temp_circ_class(icir+1) = temp_circ_class(icir)
          temp_circ_class_n(icir+1) = temp_circ_class_n(icir)
          temp_circ_class_biomass(icir+1,:,:) = &
               temp_circ_class_biomass(icir,:,:)

       ENDDO

       temp_circ_class(icir+1) = tempa
       temp_circ_class_n(icir+1) = tempb
       temp_circ_class_biomass(icir+1,:,:) = tempc(:,:)

    ENDDO
   

    !! 2.4 Merge into a single distribution with ncirc classes
    !  Merge the 2*ncirc classes in ncirc classes. Empty one circ class
    !  after the other going from the smallest to the biggest. Here we
    !  still have 2*ncirc dimensions
    circ_class_biomass(ipts,ivm,:,:,:) = zero
    trees_left(:) = temp_circ_class_n(:)
    DO icir=1,ncirc

       ! This is the number of trees in the ncirc circ classes thus
       ! AFTER the merge happened 
       trees_needed = exp_circ_class_n(icir)
       DO jcir = 1,2*ncirc

          IF (trees_needed .LT. min_stomate) EXIT

          IF (trees_left(jcir) .GE. trees_needed) THEN

             ! We can get all the trees we need from this class
             ! and don't have to continue searching. Note that
             ! the units of circ_class_biomass have temporarly 
             ! changed to gC m-2
             circ_class_biomass(ipts,ivm,icir,:,:) = &
                  circ_class_biomass(ipts,ivm,icir,:,:) + &
                  temp_circ_class_biomass(jcir,:,:) * &
                  trees_needed
             trees_left(jcir) = trees_left(jcir) - trees_needed
             trees_needed = zero

             EXIT

          ELSE

             ! The number of trees in this class are not sufficient, so
             ! we need to move all of them to the new class. Note that
             ! the units of circ_class_biomass have temporarly changed 
             ! to gC m-2
             circ_class_biomass(ipts,ivm,icir,:,:) = &
                  circ_class_biomass(ipts,ivm,icir,:,:) + &
                  temp_circ_class_biomass(jcir,:,:) * &
                  trees_left(jcir)
             trees_needed = trees_needed - trees_left(jcir)
             trees_left(jcir) = zero

          ENDIF

       ENDDO ! jcir=1,ncirc   

    ENDDO ! icir=1,ncirc


    !! 2.5 convert units back to the original units (gC tree-1) 
    !  Right now, circ_class_biomass gives the total biomass
    !  in each class, but it should only be for a model tree. 
    !  So let's normalize it. And, update the variables that 
    !  pass this information around
    DO icir=1,ncirc

       circ_class_biomass(ipts,ivm,icir,:,:) = &
            circ_class_biomass(ipts,ivm,icir,:,:) / &
            exp_circ_class_n(icir)
       circ_class_n(ipts,ivm,icir) = exp_circ_class_n(icir)         

    ENDDO
 
    IF (printlev.GE.4) WRITE(numout,*) 'Leaving merging biomass of pfts'
    
  END SUBROUTINE merge_biomass_pfts

! ================================================================================================================================
!! SUBROUTINE   : print_lcc_error_message
!!
!>\BRIEF        
!!
!! DESCRIPTION  : 
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE print_lcc_error_message(npts,ipts,veget_cov_max_new,veget_cov_max,&
       veget_cov_max_new_group,veget_cov_max_group,delta_veget_cov,loss_gain,flag)

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                                :: npts                 !! Domain size - number of pixels (unitless)
    INTEGER, INTENT(in)                                :: ipts                 !! Location of current gridsquare
    INTEGER, INTENT(in)                                :: flag                 !! Indicates what to print
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)       :: veget_cov_max            !! "maximal" coverage fraction of a PFT (LAI ->
                                                                               !! infinity) on ground.  (unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)       :: veget_cov_max_new        !! new "maximal" coverage fraction of a PFT (LAI
                                                                               !! -> infinity) on ground. 
    REAL(r_std),DIMENSION(npts,nvmap), INTENT(in)      :: delta_veget_cov          !! changes in "maximal" coverage fraction of PFT 
    REAL(r_std), DIMENSION(npts,nvmap) , INTENT(in)    :: veget_cov_max_new_group  !! Total veget_cov_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap) , INTENT(in)    :: veget_cov_max_group      !! Total veget_cov_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                   :: loss_gain            !! The same as delta_veget_cov but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)

    !! 0.2 Output variables


    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: ivm,ivma  !! Indices
!_ ================================================================================================================================

    IF(ipts .LT. 1)THEN
       WRITE(numout,*) 'ERROR: Passing a bad point to print_lcc_error_message.'
       WRITE(numout,*) 'ipts, test_grid: ',ipts,test_grid
       RETURN
    ENDIF

    SELECT CASE(flag)
    CASE(1)

       ! Case 1
       WRITE(numout,*) 'ipts ivma veget_cov_max_group veget_cov_max_new_group'
       DO ivma=1,nvmap
          WRITE(numout,'(2I7,2F14.10)') ipts,ivma,veget_cov_max_group(ipts,ivma),&
               veget_cov_max_new_group(ipts,ivma)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_cov_max_group(ipts,:)),&
            SUM(veget_cov_max_new_group(ipts,:))

    CASE(2)

       ! Case 2
       WRITE(numout,*) 'ivm veget_cov_max veget_cov_max_new'
       DO ivm=1,nvm
          WRITE(numout,'(2I7,2F14.10)') ipts,ivm,veget_cov_max(ipts,ivm),&
               veget_cov_max_new(ipts,ivm)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_cov_max(ipts,:)),&
            SUM(veget_cov_max_new(ipts,:))

    CASE(3)

       ! Case 3
       WRITE(numout,*) 'ivm veget_cov_max_group delta_veget_cov'
       DO ivma=1,nvmap
          WRITE(numout,'(2I7,3F14.10)') ipts,ivma,veget_cov_max_group(ipts,ivma),&
               delta_veget_cov(ipts,ivma)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_cov_max_group(ipts,:)),&
            SUM(delta_veget_cov(ipts,:))
    CASE(4)

       !Case 4
       WRITE(numout,*) 'ipts ivm veget_cov_max loss_gain'
       DO ivm=1,nvm
         WRITE(numout,'(2I7,3F14.10)') ipts,ivm,&
               veget_cov_max(ipts,ivm),loss_gain(ipts,ivm)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_cov_max(ipts,:)),&
            SUM(loss_gain(ipts,:))

   CASE(5)

       ! Case 5
       WRITE(numout,*) 'ivm agec_group'
       DO ivm=1,nvm
          WRITE(numout,'(2I8)') ivm,agec_group(ivm)
       ENDDO

    CASE default

       ! Default case
       WRITE(numout,*) 'ERROR: Not sure what you want me to print in LCC.'
    END SELECT
    
     
  END SUBROUTINE print_lcc_error_message


! ================================================================================================================================
!! SUBROUTINE   : adjust_delta_veget_cov
!!
!>\BRIEF        
!!
!! DESCRIPTION  :  Check whether the initially proposed changes in veget_cov_max make sense
!! If they do so accept this change. If the change brings the remaining veget_cov_max below
!! the minimal threshold remove the whole PFT. Further the maps only prescribe the change
!! in nvmap and do not contain any information about the changes in age classes. If a PFT 
!! looses surface area we assumed that each age classes looses the same relative amount 
!! of its surface area.
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE adjust_delta_veget_cov(npts,veget_cov_max,veget_cov_max_group,veget_cov_max_new,&
       veget_cov_max_new_group,loss_gain,delta_veget_cov,losses)

  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                               :: npts                  !! Domain size - number of pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvmap),INTENT(in)     :: veget_cov_max_group       !! Total veget_cov_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)      :: veget_cov_max_new         !! new "maximal" coverage fraction of a PFT (LAI
                                                                               !! -> infinity) on ground. 
    REAL(r_std), DIMENSION(npts,nvmap), INTENT(in)    :: veget_cov_max_new_group   !! Total veget_cov_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)      :: veget_cov_max             !! "maximal" coverage fraction of a PFT (LAI ->
                                                                               !! infinity) on ground.  (unitless)
    CHARACTER(15), INTENT(in)                         :: losses                !! Flag that determines how the losses are 
                                                                               !! distributed. There are currently two options
                                                                               !! 'proportional' and 'youngest'
    
    
    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)     :: loss_gain             !! The same as delta_veget_cov but distributed of all
                                                                               !! age classes and thus taking the age-classes into 


    !! 0.3 Modified variables
    REAL(r_std),DIMENSION(npts,nvmap), INTENT(inout) :: delta_veget_cov            !! changes in "maximal" coverage fraction of PFT 

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(npts,nvmap)               :: delta_veget_cov_old        !! A temporary copy of delta_veget_cov.
    INTEGER(i_std)                                   :: ivm,ivma,ipts          !! Indices
    INTEGER(i_std)                                   :: jvma,igroup,iyoung     !! Indices
    REAL(r_std)                                      :: excess_veg_add         !! 
    REAL(r_std)                                      :: excess_veg_rem         !! 
    REAL(r_std)                                      :: veg_add_tot            !! 
    REAL(r_std)                                      :: difference             !! How much leftover veget_cov_max we have
    INTEGER(i_std)                                   :: ncount                 !! The number of PFTs with significant changes
                                                                               !! in vegetation.
    INTEGER(i_std)                                   :: nagec_local            !! The number of age classes in this map PFT.
    REAL(r_std), DIMENSION(npts,nvmap)               :: temp_delta_veget_cov       !! temporary variable to handle losses and gains
                                                                               !! when there is a species change

!_ ================================================================================================================================

    !! Distribute delta_veget_cov over all age-classes
    loss_gain(:,:)=zero

    ! Here we have to be careful.  There is a variable, min_vegfrac, which checks some things
    ! below to make sure we do not end up with land that is below a certain size.  This seems to be
    ! mainly for the DGVM, but it affects us here, too.  If our final veget_cov_max is too low,
    ! this will cause problems.  Therefore, adjust them beforehand.  
    delta_veget_cov_old(:,:)=delta_veget_cov(:,:)

    DO ipts=1,npts
       
       IF(printlev.GE.4)THEN
!          IF(test_grid == ipts)THEN
             WRITE(numout,*) 'Readjusting LCC fluxes for grid: ',ipts
!          ENDIF
       ENDIF
       !  ::delta_veget_cov has been calculated for the sum of the age classes
       !  within a PFT. ::delta_veget_cov thus needs to be distributed over 
       !  all the age classes. This distribution requires a rule which 
       !  in the absence of better information is based on the assumption
       !  that a land cover change does not care about the age classes.
       !  Following a forestry logic, one would change an old forest into 
       !  a grassland because that would result in an income from the wood
       !  harvest. However, here we assume that LCC follows a spatialized 
       !  economic need and therefore does not respect the forestry logic.
       !  In other words, when a farmer needs more land, he will not cut
       !  spatialy distributed old stands but will target a block of land
       !  bordering his current agricultural lands. By doing so forest
       !  of all age classes will be cut when LCC is the underlying 
       !  motivation. The reasoning applies for deforestation driven by 
       !  urbanisation. This assumption translates in that the land cover
       !  fraction of all age classes will decrease proportionally to their
       !  share when the PFT experiences a loss. When the PFT experiences an
       !  increase in the PFT cover this only happens in the youngest age
       !  class.

       ! There is one more constraint.  The new vegetation fraction at
       ! the end of the calculation must be larger than min_vegfrac.  It might
       ! not be if we remove the wrong amount of vegetation.  If veget_cov_max
       ! is greater than zero but less than min_vegfrac after accounting
       ! for loss_gain, we kill the whole PFT and move the leftover to
       ! bare soil.  This leftover should be a small number.
 
       ! This is a clean up case.  We should not have absolute values of loss_gain
       ! or delta_veget_cov which are less than min_stomate.  This carbon will get
       ! lost otherwise.  First let's do delta_veget_cov.  We will have to do something
       ! at the end to be sure nothing has gone wrong with loss_gain, too.
       excess_veg_add=zero
       excess_veg_rem=zero

       DO ivma=1,nvmap

          IF ((delta_veget_cov(ipts,ivma) .LT. min_stomate) .AND. &
               (delta_veget_cov(ipts,ivma) .GT. zero)) THEN

             IF(printlev.GE.4 .AND. ipts == test_grid)THEN
                WRITE(numout,*) 'delta_veget_cov between zero and min_stomate, adding ',&
                     ivma,delta_veget_cov(ipts,ivma)
             ENDIF

             ! We want to add just a tiny amount of land. We will not do this as
             ! as most likely it is just a precision issue. Add the land cover 
             ! change to a temporary variable which will be dealt with later.
             excess_veg_add=excess_veg_add+delta_veget_cov(ipts,ivma)
             delta_veget_cov(ipts,ivma)=zero

          ELSEIF((delta_veget_cov(ipts,ivma) .LT. zero) .AND. &
               (delta_veget_cov(ipts,ivma) .GT. -min_stomate))THEN
             
             IF(printlev.GE.4 .AND. ipts == test_grid)THEN
                WRITE(numout,*) 'delta_veget_cov between zero and -min_stomate, removing ',&
                     ivma,delta_veget_cov(ipts,ivma)
             ENDIF

             ! We want to remove just a tiny amount of land. We will not do this as
             ! as most likely it is just a precision issue. Remove the land cover 
             ! change from a temporary variable which will be dealt with later.
             excess_veg_rem=excess_veg_rem+delta_veget_cov(ipts,ivma)
             delta_veget_cov(ipts,ivma)=zero

          ENDIF

       ENDDO


       ! Make use of temporary variable so that the values can be overwritten
       temp_delta_veget_cov = delta_veget_cov

       ! From here on, we are only concerned with loss_gain.  We will ignore
       ! delta_veget_cov after creating loss_gain, and then we will reconstruct
       ! delta_veget_cov at the end from the changes we have made to loss_gain.
       ! Note that loss_gain has the dimensions of the simulation (::nvm) 
       ! whereas delta_veget_cov has the dimension of the maps (::nvma)
       DO ivm=1,nvm

          igroup=agec_group(ivm)
          iyoung=start_index(igroup)

          ! Only forest PFTs have age classes
          IF (is_tree(ivm)) THEN

             ! PFT cover area decreases (loss)
             IF (temp_delta_veget_cov(ipts,igroup) .LE. -min_stomate) THEN

                SELECT CASE(losses)
                CASE('proportional')

                   ! When there is a loss in veget_cov_max the standard code
                   ! will distribute this loss proportionaly to the populated
                   ! age classes of the species group. This reflects the
                   ! fact that a land cover change does not necessarily 
                   ! happens at the end of a forest rotation. All, young, 
                   ! medium and old forest will be deforested.
                   loss_gain(ipts,ivm) = veget_cov_max(ipts,ivm) / &
                        veget_cov_max_group(ipts,igroup) * temp_delta_veget_cov(ipts,igroup)

                CASE('youngest')

                   ! This option is intended for species change only.
                   ! A species change only happens at the end of a 
                   ! rotation. So it is an old age class that will be
                   ! replaced by a young one. Note that in forestry the
                   ! old age class is marked for killing. In kill the
                   ! surface of the harvested age class is moved to
                   ! the youngest age class and here we move it 
                   ! from the youngest age class of the previous species
                   ! to the youngest age class of the new species.
                   IF(veget_cov_max(ipts,ivm) .GT. min_stomate) Then
                    loss_gain(ipts,ivm) = temp_delta_veget_cov(ipts,igroup)
                    temp_delta_veget_cov(ipts,igroup) = zero
                   ENDIF

                CASE DEFAULT

                   ! Stop the simulation
                   CALL ipslerr_p (3,'land_cover_main', 'No information on',&
                     'how the losses in veget_cov_max should be distributed','')

                END SELECT

             ! PFT cover area increases (gain)
             ELSE

                ! The first age-class in a PFT group is the youngest
                ! and that is the age-class were the change in cover
                ! fraction takes place
                IF(ivm == iyoung)THEN

                   loss_gain(ipts,ivm) = temp_delta_veget_cov(ipts,igroup)

                ELSE

                   ! The remaining age-classes in that PFT don't experience a
                   ! change in cover fraction  
                   loss_gain(ipts,ivm) = zero

                ENDIF ! looking for youngest age class

             ENDIF

          ! Grasses and croplands
          ELSE

             ! There are no age-classes for grasses and crops, hence, 
             ! delta_veget_cov should not be distributed over the age classes
             loss_gain(ipts,ivm) = temp_delta_veget_cov(ipts,igroup)

          ENDIF ! is_tree(ivm)

       ENDDO ! looping over PFTs

       IF (printlev.GE.4) THEN
!          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'First estimate of loss_gain '
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)
!          ENDIF
       ENDIF

       ! It is possible that we try to add a very small amount to a PFT
       ! which does not exist.  We cannot have a veget_cov_max between
       ! min_stomate and min_vegfrac.  We will remove such small
       ! changes here, but we need to be sure to add them somewhere else.
       DO ivm=1,nvm

          IF(veget_cov_max(ipts,ivm) .LT. min_stomate)THEN

             IF((loss_gain(ipts,ivm) .GT. zero) .AND. &
                  (loss_gain(ipts,ivm) .LE. min_vegfrac))THEN

                IF(printlev.GE.4)THEN

                   WRITE(numout,*) 'Trying to add too small of loss_gain to a PFT ' // &
                        'which does not exist.  Correcting.'
                   WRITE(numout,*) 'ipts, ivm, min_vegfrac: ',ipts, ivm, min_vegfrac
                   WRITE(numout,*) 'veget_cov_max,loss_gain (before): ',&
                        veget_cov_max(ipts,ivm),loss_gain(ipts,ivm)
                ENDIF

                ! Loss_gain is too small. Ignore it but make
                ! sure to account for it somewhere else. If
                ! not the C balance will not be closed.
                excess_veg_add = excess_veg_add+loss_gain(ipts,ivm)
                loss_gain(ipts,ivm)=zero

             ENDIF
                
          ENDIF

       ENDDO ! looping over all PFTs

       ! This is a first readjustment due to the fact that creating
       ! loss_gain proportational to veget_cov_max might generate a
       ! new veget_cov_max for a given PFT which is below min_fracveg.
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Only a concern if we have age classes
          IF (nagec_local .GT. 1) THEN

             ! Will any veget_cov_maxes created by these loss_gains
             ! be below the minimum vegetation fraction?
             DO ivm=iyoung,iyoung+nagec_local-1

                IF(loss_gain(ipts,ivm) .LT. zero)THEN

                   ! We also need to check if it's greater than zero, 
                   ! since we might want to remove the whole PFT, 
                   ! which is perfectly okay.
                   IF(veget_cov_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)) .LE. min_vegfrac .AND. &
                        (veget_cov_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)) .NE. zero))THEN

                      IF(printlev.GE.4)THEN
                         WRITE(numout,*) 'Results in a veget_cov_max below min_vegfrac!  Correcting.'
                         WRITE(numout,*) 'ipts, ivm, min_vegfrac: ',ipts, ivm, min_vegfrac
                         WRITE(numout,*) 'veget_cov_max,loss_gain (before): ',&
                              veget_cov_max(ipts,ivm),loss_gain(ipts,ivm)
                      ENDIF

                      ! We kill the whole PFT.  This means we have to add a gain somewhere
                      ! to compensate, probably bare soil.
                      excess_veg_add = excess_veg_add + &
                           (veget_cov_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)))
                      loss_gain(ipts,ivm)=-veget_cov_max(ipts,ivm)

                   ENDIF

                ENDIF ! loss_gain .LT.zero

             ENDDO ! ivm

          ENDIF ! checking if we have age classes 

       ENDDO ! looping over the map PFTs

       ! Now we do one final check to make sure that our loss_gain isn't too small.
       ! It should be fairly rare that we actually do something here.  We have
       ! already done the same check above for delta_veget_cov, so if we don't have
       ! age classes there will be nothing to do here.  This is required because
       ! some of the actions in LCC are not done if loss_gain is below min_stomate,
       ! so we would have a mass balance error otherwise.
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Only a concern if we have age classes
          IF (nagec_local .GT. 1) THEN

             ! Will any veget_cov_maxes created by these loss_gains
             ! be below the minmum vegetation fraction?
             DO ivm=iyoung,iyoung+nagec_local-1

                IF ((loss_gain(ipts,ivm) .LT. min_stomate) .AND. &
                     (loss_gain(ipts,ivm) .GT. zero)) THEN
                   
                   IF(printlev.GE.4)THEN
                      WRITE(numout,*) 'loss_gain between zero and min_stomate, adding ',&
                           ivm,ivma,delta_veget_cov(ipts,ivma),loss_gain(ipts,ivm)
                   ENDIF

                   ! We want to add just a tiny amount of land.  
                   ! Add it somewhere else.
                   excess_veg_add=excess_veg_add+loss_gain(ipts,ivm)
                   loss_gain(ipts,ivm)=zero
                   
                ELSEIF((loss_gain(ipts,ivm) .LT. zero) .AND. &
                     (loss_gain(ipts,ivm) .GT. -min_stomate))THEN
                   
                   IF(printlev.GE.4)THEN
                      WRITE(numout,*) 'loss_gain between zero and -min_stomate, removing ',&
                           ivm,ivma,delta_veget_cov(ipts,ivma),loss_gain(ipts,ivm)
                   ENDIF

                   ! We want to add just a tiny amount of land.  
                   ! Add it somewhere else.
                   excess_veg_rem=excess_veg_rem+loss_gain(ipts,ivm)
                   loss_gain(ipts,ivm)=zero
                   
                ENDIF

             ENDDO ! loop over the age classes for this PFT

          ENDIF ! checking if we have age classes 

       ENDDO ! looping over the map PFTs


       ! If we have extra land to add/remove, let's change the loss_gain of bare soil.
       ! This should, in general, be a small number.
       IF (printlev.GE.4) THEN
!          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'Extra vegetation to add: '
             WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
             WRITE(numout,*) 'Extra vegetation to remove: '
             WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem

             WRITE(numout,*) 'Readjusting loss_gain to make sure an age class does not fall too low.'

             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new,veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)

!          ENDIF

       ENDIF

       ! Our loss_gain should not lead to a bare soil fraction below min_vegfrac.
       ! Start with PFT 1 (bare soil) and try to put all the excess into a single PFT.
       ! If we cannot do that, throw an error. It's possible that we can flip the sign
       ! of the loss_gain by doing this, but that should not cause any
       ! serious problem.  We just won't be 100% inline with the maps, but as long
       ! as min_vegfrac exists, we won't be anyways.

       ! We loop over the map PFTs here.  We don't want to loop over all the PFTs
       ! and check, because we might end up adding area to an age class which
       ! is not the youngest that way, which would be bad.
       excess:DO ivma=1,nvmap

          ! Do we have vegetation?
          ivm=start_index(ivma)

          IF(veget_cov_max(ipts,ivm) .GE. min_stomate)THEN

             ! Can we add the difference without falling below min_vegfrac?
             IF((veget_cov_max(ipts,ivm)+(loss_gain(ipts,ivm)+&
                  excess_veg_add+excess_veg_rem)) .GT. min_vegfrac)THEN

                ! Also, our total loss_gain needs to be above min_stomate
                IF(ABS(loss_gain(ipts,ivm)+&
                     excess_veg_add+excess_veg_rem) .GT. min_stomate)THEN
                   
                   IF(printlev.GE.4)THEN
                      WRITE(numout,*) 'Dumping excess vegetation'
                      WRITE(numout,*) 'ipts,ivm,ivma: ',ipts,ivm,ivma
                      WRITE(numout,*) 'Correction: ',&
                           excess_veg_add+excess_veg_rem
                      WRITE(numout,*) 'veget_cov_max: ',veget_cov_max(ipts,ivm)
                      WRITE(numout,*) 'veget_cov_max corrected: ',veget_cov_max(ipts,ivm)+&
                           (loss_gain(ipts,ivm)+excess_veg_add+excess_veg_rem)
                      WRITE(numout,*) 'loss_gain: ',loss_gain(ipts,ivm)
                      WRITE(numout,*) 'loss_gain corrected: ',loss_gain(ipts,ivm)+&
                           excess_veg_add+excess_veg_rem
                   ENDIF

                   ! Let's do it.
                   loss_gain(ipts,ivm)=loss_gain(ipts,ivm)+&
                        excess_veg_add+excess_veg_rem
                   excess_veg_add=zero
                   excess_veg_rem=zero

                   EXIT excess
                ENDIF
             ENDIF
          ENDIF
       ENDDO excess

       ! If we still have excess vegetation, our method did not work.
       IF((excess_veg_add .GE. min_stomate) .OR. &
            (ABS(excess_veg_rem) .GE. min_stomate))THEN

          ! It is possible that the excess amount we have to add 
          ! and that which we have to remove cancel each other 
          ! out.  In which case, we don't actually have a problem.
          IF(ABS(excess_veg_add+excess_veg_rem) .GE. min_stomate)THEN

             WRITE(numout,*) 'We were not able to redistribute all the extra vegetation in LCC.'
             WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
             WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem
             WRITE(numout,*) 'ipts: ',ipts
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new,veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)

             ! Ignore this problem (which is a useful flag when
             ! going from the spin-up to a transient simulation
             IF(err_act.GT.1)THEN
                CALL ipslerr_p (2,'land_cover_main', 'Not stopping here.',&
                     'Only ignore me if you know what you are doing!','Place 2')
                excess_veg_add=zero
                excess_veg_rem=zero

             ELSE

                CALL ipslerr_p (3,'land_cover_main', &
                     'Unable to redistribute vegetation (see output file)','','')
             ENDIF

             
          ELSE ! Success redistribution

             IF(printlev.GE.4)THEN
                WRITE(numout,*) 'All extra vegetation successfully redistributed in LCC.'
                WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
                WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem
             ENDIF

             excess_veg_add=zero
             excess_veg_rem=zero

          ENDIF

       ENDIF

       ! Reconstruct the delta_veget_cov from the adjusted loss_gain array.
       delta_veget_cov(ipts,:)=zero
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Add all these age classes to this map PFT
          DO ivm=iyoung,iyoung+nagec_local-1

             delta_veget_cov(ipts,ivma)=delta_veget_cov(ipts,ivma)+loss_gain(ipts,ivm)

          ENDDO

       ENDDO ! done looping over map PFTs


       ! Now one final sanity check.  We should never try to remove more
       ! vegetation than we actually have.  
       DO ivm=1,nvm

          ! Notice here that loss_gain is negative for pixels
          ! which remove veget_cov_max.
          difference=veget_cov_max(ipts,ivm)+loss_gain(ipts,ivm)

          IF(difference .LT. -min_stomate)THEN

             ! This is a problem.
             WRITE(numout,*) 'We are trying to remove too much vegetation in LCC.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             WRITE(numout,*) 'veget_cov_max, loss_gain: ',&
                  veget_cov_max(ipts,ivm),loss_gain(ipts,ivm)
             CALL ipslerr_p (3,'land_cover_main', &
                  'Removing too much vegetation (see output file)','','')
             
          ELSEIF((difference .LE. min_vegfrac) .AND. &
               (difference .GT. min_stomate))THEN

             ! We should have accounted for this above.
             WRITE(numout,*) 'We are left with an awkward amount of vegetation in LCC.'
             WRITE(numout,*) 'This should have been taken care of above.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             CALL print_lcc_error_message(npts,ipts,veget_cov_max_new,&
                  veget_cov_max,veget_cov_max_new_group,&
                  veget_cov_max_group,delta_veget_cov,loss_gain,4)
             CALL ipslerr_p (3,'land_cover_main', &
                  'Awkward amoutn of vegetation (see output file)','','')
          
          ELSEIF(difference .LT. zero)THEN
          
             ! Technically not a problem since it could be -1e-17, but it might provoke
             ! a mass balance error so we warn about it.  The serious case of
             ! a difference less than -min_stomate was handled above.
             WRITE(numout,*) 'WARNING: A small amount of leftover vegatation in LCC.'
             WRITE(numout,*) 'Should not cause a problem, but watch for a mass balance error.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             WRITE(numout,*) 'veget_cov_max, loss_gain: ',&
                  veget_cov_max(ipts,ivm),loss_gain(ipts,ivm)
             CALL ipslerr_p (2,'land_cover_main', &
                  'Small amount of leftover vegetation (see output file)','','')
          ENDIF

       ENDDO ! loop over PFTs

    ENDDO ! loop over #pixels - domain size

  END SUBROUTINE adjust_delta_veget_cov

END MODULE sapiens_lcchange
